'use strict';

const jsdom = require('jsdom');

class DomTest {
  constructor() {}

  _init(options) {
    this.errorCounter = 0;
    this.runtimeErrors = [];
    this.resultCode = 0;
    this.virtualConsole = jsdom.createVirtualConsole();
    this.virtualConsole.on("jsdomError", (error) => {this._onJsdomError(error);});
    this.jsdomConfig = {
      url: "",
      virtualConsole: this.virtualConsole,
      created: (error, window) => {this._onCreated(error, window);},
      done: (error, window) => {this._onDone(error, window);},
      strictSSL: false,
      features: {
        FetchExternalResources: ["script", "link"],
        ProcessExternalResources: ["script"],
        // SkipExternalResources: /https?:\/\/(?!localhost)/
      },
      // resourceLoader: (resource, callback) => {
      //   console.log(`loading ${resource.url.host}${resource.url.pathname} ...`);
      //   return resource.defaultFetch(callback);
      // },
    };
    this.options = options;
  }

  _onJsdomError(error) {
    // console.log("message:" + error.detail.message);
    var ignore = false;
    if (this.options.ignoreMessages
        && this.options.ignoreMessages.forEach) {
      this.options.ignoreMessages.forEach(
        (val, index, array) => {
          const rex = new RegExp(val);
          if (error.detail.message.match(rex)) {
            ignore = true;
          }
        }
      );
    }
    if (ignore) { return; }

    this.runtimeErrors.push(error);
    this.errorCounter++;
  }

  _onDone(error, window) {
    if (!this.doneTestCallback) {
      return;
    }

    if (error) {
      this.doneTestCallback(error, this);
      return;
    }

    window.setTimeout(() => {
      if (this.errorCounter) {
        this.resultCode = 1;
      }
      this.doneTestCallback(error, this);
      window.close();
    }, this.options.finishTimeout || 5000);
  }

  _onCreated(error, window) {
    if (error) {
      this.doneTestCallback(error, this);
      return;
    }
  }

  run(url, options, callback) {
    this._init(options);
    this.jsdomConfig.url = url;
    this.doneTestCallback = callback;
    jsdom.env(this.jsdomConfig);
  }
}

module.exports = DomTest;
