package Newslash::Command;
use Mojo::Base 'Mojolicious::Command';
use List::Util qw(all);

has experimental => 0;
has danger => 0;


sub check_force_option {
    my ($self, $args) = @_;

    if ($self->experimental) {
        my $mode = $self->app->mode;
        if (!$mode || $mode ne "test") {
            if (all { $_ !~ /^\s*-f\s*$/ } @$args) {
                $self->app->log->error("This command is experimental, so needs '-f' option under '$mode' mode.");
                return;
            }
            @$args = grep { $_ !~ /^\s*-f\s*$/ } @$args;
	    return 1 if $self->danger;
        }
    }

    if ($self->danger) {
        my $mode = $self->app->mode;
        if (!$mode || $mode ne "test") {
            if (all { $_ !~ /^\s*-f\s*$/ } @$args) {
                $self->app->log->error("This command is dangerous, so needs '-f' option under '$mode' mode.");
                return;
            }
            @$args = grep { $_ !~ /^\s*-f\s*$/ } @$args;
        }
    }

    return 1;
}

1;

=encoding utf8

=head1 NAME

Newslash::Command - newslash administer command base class

=head1 ATTRIBUTES

L<Newslash::Command> inherits all attributes from
L<Mojolicious::Command> and implements the following new ones.

=head2 experimental

  my $experimental = $daemon->experimental;

Experimental flag that indicate experimental command.

=head2 danger

  my $danger = $daemon->danger;

Danger flag that indicate danger (destructive) command.

=head1 METHODS

L<Newslash::Command> inherits all methods from
L<Mojolicious::Command> and implements the following new ones.

=head2 check_options

  $daemon->check_options(\@args);

Check if this command is danger/experimental, then check "-f" option given.
if command is danger and "-f" option not given, return false.
if command is experimental and mode is not "test" and "-f" option not given, return false.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.

=cut
