package Newslash::Command::configdump;
use Mojo::Base 'Newslash::Command';

use Mojo::Util 'getopt';
use Mojo::File;
use Data::Dumper;
use Encode;
use YAML qw(Dump);
use utf8;

has description => 'dump config file';
has usage => sub { shift->extract_usage };

has experimental => 0;
has danger => 0;

sub run {
    my ($self, @args) = @_;

    my $filename = shift @args;
    my $err;

    $err = "output file not given" if !$filename;
    $err = "invalid option: $filename" if $filename && $filename =~ m/^-/;
    if ($err) {
	$self->app->log->error("configdump: $err");
	return;
    }

    my $conf = $self->app->config;
    my $yaml = Dump($conf);

    $yaml =~ s/^(\S.*:)$/\n$1/gm;
    utf8::encode($yaml);
    my $path = Mojo::File::path($filename);
    $path->spurt($yaml);
}


1;

=encoding utf8

=head1 NAME

Newslash::Command::configdump - newslash administer command (configdump)

=head1 SYNOPSIS

  Usage: APPLICATION configdump <output file>

      ./newslash_web configdump config_sample.yaml

=head1 DESCRIPTION

  L<Newslash::Command::configdump> dump current configuration to file with YAML format.

=head1 ATTRIBUTES

L<Newslash::Command::configdump> inherits all attributes from
L<Newslash::Command> and implements the following new ones.

=head2 description

  my $description = $daemon->description;
  $daemon         = $daemon->description('Foo');

Short description of this command, used for the command list.

=head2 usage

  my $usage = $daemon->usage;
  $daemon   = $daemon->usage('Foo');

Usage information for this command, used for the help screen.

=head1 METHODS

L<Newslash::Command::configdump> inherits all methods from
L<Newslash::Command> and implements the following new ones.

=head2 run

  $daemon->run(@ARGV);

Run this command.

=head1 SEE ALSO

L<Mojolicious>, L<Mojolicious::Guides>, L<https://mojolicious.org>.

=cut
