package Newslash::Eventd;
use Mojo::Base 'Mojolicious';

use Newslash::Model;

use constant CONFIG_FILE => '/etc/newslash/newslash.conf';

# This method will run once at server start
sub startup {
    my $app = shift;


    # load config file
    # first, check existence of /etc/newslash.conf
    if ($app->mode eq 'production' && -e CONFIG_FILE) {
        $app->plugin('Newslash::Plugin::YAMLConfig', file => CONFIG_FILE);
    }
    else {
	$app->plugin('Newslash::Plugin::YAMLConfig');
    }
    # TODO: load/save configs with database

    my $eventd_conf = $app->config->{Eventd} ||= {};

    # load default configuration values
    $app->plugin('Newslash::Plugin::DefaultConfig');

    # when "test" mode, output debug logs.
    $app->log->level('debug') if $app->mode eq 'test';

    if ($eventd_conf->{log_level}) {
        if ($eventd_conf->{log_level} =~ m/^(debug|info|warn|error|fatal)$/) {
            $app->log->level($eventd_conf->{log_level});
            $app->log->info("Eventd::startup: set log_level to '$eventd_conf->{log_level}'");
        }
        else {
            $app->log->error("Eventd::startup: invalid log_level ($eventd_conf->{log_level})");
        }
    }

    # secret key for hasing
    $app->secrets([$app->config->{System}->{secret_key},]);

    # support BasicAuth
    $app->plugin('Newslash::Plugin::BasicAuth');

    # use TimeLimitedCache ($app->cache)
    $app->plugin('Newslash::Plugin::TimeLimitedCache');

    # use KeyValue Store ($app->kvs)
    $app->plugin('Newslash::Plugin::KeyValueStore');

    # add Model Loader
    $app->plugin('Newslash::Plugin::Model');

    # use Template::Toolkit 2 render
    $app->plugin('Newslash::Plugin::TT2Renderer');

    # Users helper
    $app->plugin('Newslash::Plugin::Users');

    # Sendmail helper
    $app->plugin('Newslash::Plugin::Sendmail');

    # Router
    my $r = $app->routes;

}

1;
