package Newslash::Model::Globjs;
use Newslash::Model::Base -base;

use Data::Dumper;
use Mojo::Log;

my $_gtid_of = {
               stories => 1,
               urls => 2,
               submissions => 3,
               journals => 4,
               comments => 5,
               discussions => 7,
               preview => 8,
               projects => 11,
               tagnames => 17,

               story => 1,
               url => 2,
               submission => 3,
               journal => 4,
               comment => 5,
               discussion => 7,
               preview => 8,
               project => 11,
               tagname => 17,
              };

sub on_start_up {
    my $self = shift;

    # check if data is already stored in 'globj_types' table
    my $globj_types = $self->getGlobjTypes;
    return if $self->check_readonly;
    my $log = Mojo::Log->new;

    if (!$globj_types) {
        $log->warn("Globjs: globj_types retrieve failed.");
        return 1;
    }

    if (!%$globj_types) {
        # insert datas
        $log->info("no globj_types, create them.");
        my $sql = <<"EOSQL";
INSERT INTO `globj_types`
  VALUES
         (1,'stories'),
         (2,'urls'),
         (3,'submissions'),
         (4,'journals'),
         (5,'comments'),
         (7,'discussions'),
         (8,'preview'),
         (11,'projects'),
         (17,'tagnames')
EOSQL
        my $dbh = $self->connect_db;
        my $rs = $dbh->do($sql, undef);
        $self->disconnect_db;
        if ($rs) {
            return 1;
        }
        $self->warn("cannot insert data to globj_types...");
        return;
    }
    return 1;
}

sub get_gtid {
    my ($self, $type) = @_;
    return $_gtid_of->{$type};
}

sub get_globjid_from_item {
    my ($self, $type, $id) = @_;
    my $gtid = $self->get_gtid($type);
    return if !$gtid;

    my $sql = <<"EOSQL";
SELECT * FROM globjs WHERE gtid = ? AND target_id = ?
EOSQL

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute($gtid, $id);
    my $rs = $sth->fetchall_arrayref({});
    if (!$rs) {
        $self->disconnect_db;
        return;
    }

    $self->disconnect_db;
    if (@$rs > 0) {
        return $rs->[0]->{globjid};
    }
    return;
}

sub create {
    my ($self, $maintable, $target_id) = @_;
    return if $self->check_readonly;

    if (!$maintable || !$target_id) {
        $self->set_error('invalod argument', 1);
        return;
    }
    my $gtid = $self->get_gtid($maintable);
    if (!$gtid) {
        $self->set_error('invalid maintable', 1);
        return;
    }

    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
INSERT INTO globjs
    (gtid, target_id)
  VALUES
    (?,    ?)
EOSQL

    my $globj_id;
    my $rs = $dbh->do($sql, undef, $gtid, $target_id);
    if ($rs) {
        $globj_id = $dbh->last_insert_id(undef, undef, undef, undef);
    } else {
        $self->set_errorno($dbh->{mysql_errno});
    }

    $self->disconnect_db;

    if (!$globj_id) {
        return;
    }
    return $globj_id;
}



## WORNING:
## This is worst legacy from slash. deprecated, and you can use only backword-compatible.

sub getGlobjidCreate {
    my($self, $name, $target_id, $options) = @_;
    my $globjid = $self->getGlobjidFromTargetIfExists($name, $target_id);
    return $globjid if $globjid;
    return $self->createGlobjid($name, $target_id);
}

sub getGlobjidFromTargetIfExists {
    my ($self, $name, $target_id) = @_;
    return if $self->check_readonly;
    return 0 if !$name || !$target_id;

    my $globjtypes = $self->getGlobjTypes();
    my $gtid = $globjtypes->{$name};
    return 0 if !$gtid;

    #my $dbh = $self->connect_db({AutoCommit => 0,});
    my $dbh = $self->connect_db();
    my $sql = "SELECT * from globjs WHERE gtid = ? AND target_id = ?";
    my $sth = $dbh->prepare($sql);
    $sth->execute($gtid, $target_id);
    my $rs = $sth->fetchall_arrayref({});

    if (@$rs == 0) {
        return;
    }
    return $rs->[0]->{globjid};
}

sub createGlobjid {
    my ($self, $name, $target_id) = @_;
    return if $self->check_readonly;
    my $globjtypes = $self->getGlobjTypes;
    my $gtid = $globjtypes->{$name};
    return 0 if !$gtid;

    my $dbh = $self->connect_db();
    my $sql = "INSERT INTO globjs (gtid, target_id) VALUES (?, ?)";
    my $result = $dbh->do($sql, undef, $gtid, $target_id);
    if (!$result) {
        return $self->getGlobjidFromTargetIfExists($name, $target_id);
    }
    my $globjid = $dbh->last_insert_id(undef, undef, undef, undef);
    return $globjid;
}

# Returns a hashref in which the keys are either numeric gtid's OR
# the names of maintables, and the values are the opposite.  So if
# the globj_types table consists of the single row (1, 'stories'),
# then this method will return the hashref: { stories => '1',
# '1' => 'stories' }.
sub getGlobjTypes {
    my ($self) = @_;

    #my $dbh = $self->connect_db({AutoCommit => 0,});
    my $dbh = $self->connect_db();
    my $sql = "SELECT * from globj_types";
    my $sth = $dbh->prepare($sql);

    eval {
        $sth->execute();
    };
    if ($@) {
        $self->disconnect_db;
        return;
    }
    my $rs = $sth->fetchall_arrayref({});

    my $result = {};
    for my $item (@$rs) {
        $result->{$item->{gtid}} = $item->{maintable};
        $result->{$item->{maintable}} = $item->{gtid};
    }
    $sth->finish;
    $self->disconnect_db;
    return $result;
}

# Given a globjid, returns its globj_type and target_id.  Returns
# undef if the object does not exist.
# XXX should memcached
sub getGlobjTarget {
    my ($self, $globjid) = @_;
    my ($gtid, $target_id) = $self->sqlSelect(
                                              'gtid, target_id',
                                              'globjs',
                                              "globjid=$globjid");
    return undef unless $gtid;
    my $types = $self->getGlobjTypes;
    return undef unless $types->{$gtid};
    return ($types->{$gtid}, $target_id);
}

# Given an arrayref of globjids, returns a hashref where each key is a
# globjid and its value is an arrayref of its globj_type,target_id.
# XXX should memcached
sub getGlobjTargets {
    my ($self, $globjid_ar) = @_;
    return { } if !$globjid_ar || !@$globjid_ar;

    my $target_hr = { };
    my $in_str = join(',', grep /^\d+$/, @$globjid_ar);
    my $ar_ar = $self->sqlSelectAll('globjid, gtid, target_id', 'globjs', "globjid IN ($in_str)");
    my $types = $self->getGlobjTypes;
    for my $ar (@$ar_ar) {
        $target_hr->{ $ar->[0] } = [ $types->{ $ar->[1] }, $ar->[2] ];
    }
    return $target_hr;
}

# Returns the string associated with a single globj's admin note.
# XXX should memcached
sub getGlobjAdminnote {
    my ($self, $globjid) = @_;
    return undef if !$globjid;
    my $globjid_q = $self->sqlQuote($globjid);
    return $self->sqlSelect('adminnote',
                            'globj_adminnotes',
                            "globjid=$globjid_q");
}

# Returns a hashref for multiple globjs' admin notes:  key is the
# globjid, value is the string of the note.
# sub getGlobjAdminnotes {
#     my ($self, $globjid_ar) = @_;
#     return { } if !$globjid_ar || !@$globjid_ar;
#     my $in_clause = join(',', map { $self->sqlQuote($_) } @$globjid_ar);
#     return $self->sqlSelectAllKeyValue(
#                                        'globjid, adminnote',
#                                        'globj_adminnotes',
#                                        "globjid IN ($in_clause)");
# }

sub getGlobjAdminnotes {
    my ($self, $globj_ids) = @_;
    if (!$globj_ids || !@$globj_ids) {
        return {};
    }
    my $in_clause = join(", ", map { "?" } @$globj_ids);
    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
SELECT globjid, adminnote FROM globj_adminnotes
  WHERE globjid IN ($in_clause)
EOSQL
    my $sth = $dbh->prepare($sql);
    $sth->execute(@$globj_ids);
    my $rs = $sth->fetchall_hashref('globjid');
    $self->disconnect_db;
    if (!$rs) {
        return {};
    }

    my $ret = {};
    for my $k (keys %$rs) {
        $ret->{key} = $rs->{key}->{adminnote};
    }
    return $ret;
}



# Sets the admin note associated with a single globj.  If undef
# or the empty string are passed in for the note, the row is
# removed from the table (and undef will be returned if the
# note is later requested).
sub setGlobjAdminnote {
    my($self, $globjid, $note) = @_;
    return 0 if !$globjid;
    my $db = $self->new_instance_of('LegacyDB');
    if (defined($note) && length($note)) {
        #return $self->sqlReplace(
        return $db->sqlReplace(
                               'globj_adminnotes',
                               { globjid => $globjid, adminnote => $note });
    }
    #my $globjid_q = $self->sqlQuote($globjid);
    #return $self->sqlDelete('globj_adminnotes', "globjid = $globjid_q");
    my $globjid_q = $db->sqlQuote($globjid);
    return $db->sqlDelete('globj_adminnotes', "globjid = $globjid_q");
}

1;
