package Newslash::Model::Timeline;
use Newslash::Model::Base -base;

use Data::Dumper;


sub select {
    my $self = shift;
    my $params = {@_};

    my $uid = $params->{uid};
    my $type = $params->{type} || "global";
    if ($uid) {
        $type = $params->{type} || "user";
    }

    if ($type eq "user") {
        return $self->_select_user($uid, $params);
    }
    elsif ($type eq "tags") {
        return $self->_select_tags($uid, $params);
    }
    elsif ($type eq "user") {
        return $self->_select_user($uid, $params);
    }
    elsif ($type eq "friends") {
        return $self->_select_friends($uid, $params);
    }
    return $self->_select_global($params);
}

sub _select_tags {
    my ($self, $items) = @_;
    return if !$items;

    return 1 if !@$items;

    my @globj_ids = map { $_->{globjid} } @$items;
    my @placeholders = map { "?" } @globj_ids;
    my $placeholder = join(", ", @placeholders);

    my $dbh = $self->connect_db;
    my $sql = <<"EOSQL";
SELECT * FROM tags
  LEFT JOIN tagnames ON tags.tagnameid = tagnames.tagnameid
  WHERE tags.globjid IN ($placeholder)
EOSQL

    my $sth = $dbh->prepare($sql);
    $sth->execute(@globj_ids);
    my $rs = $sth->fetchall_arrayref({});
    $self->disconnect_db;

    return if !$rs;

    my $tags_of = {};
    for my $tag (@$rs) {
        my $id = $tag->{globjid};
        $tags_of->{$id} = [] if !$tags_of->{$id};
        push @{$tags_of->{$id}}, $tag;
    }

    for my $item (@$items) {
        my $globj_id = $item->{globjid};
        $item->{tags} = $tags_of->{$globj_id} || [];
    }

    return 1;
}

# select global timeline
sub _select_global {
    my ($self, $params) = @_;

    my $unique_keys = { id => "id" };
    my $keys = { popularity => "popularity",
               };
    my $datetime_keys = { create_time => "createtime",
                          update_time => "last_update",
                        };
    my $timestamp = "createtime";

    my ($where_clause, $where_values, $unique) = $self->build_where_clause(unique_keys => $unique_keys,
                                                                           keys => $keys,
                                                                           datetime_keys => $datetime_keys,
                                                                           timestamp => $timestamp,
                                                                           params => $params);
    my ($limit_clause, $limit_values) = $self->build_limit_clause(params => $params);
    my ($orderby_clause, $orderby_values) = $self->build_order_by_clause(keys => $keys,
                                                                         params => $params);

    if ($where_clause) {
        $where_clause = $where_clause . " AND public = 'yes' AND createtime <= NOW()";
    }
    else {
        $where_clause = "WHERE public = 'yes' AND createtime <= NOW()";
    }

    if ($orderby_clause) {
        $orderby_clause = $orderby_clause . ", createtime DESC";
    }
    else {
        $orderby_clause = " ORDER BY createtime DESC";
    }

    my $sql = <<"EOSQL";
SELECT fh.*,
       firehose_text.*,
       topics.*,
       stories.sid,
       users.nickname AS author
  FROM (SELECT * FROM firehose
          $where_clause
          $orderby_clause
          $limit_clause)
    AS fh
  JOIN firehose_text ON fh.id = firehose_text.id
  JOIN users ON fh.uid = users.uid
  LEFT JOIN stories ON fh.type = "story" AND fh.srcid = stories.stoid
  LEFT JOIN topics ON fh.tid = topics.tid
EOSQL
    my @params;
    push @params, @$where_values;
    push @params, @$orderby_values;
    push @params, @$limit_values;

    #warn $sql;
    #warn Dumper @params;

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute(@params);
    my $rs = $sth->fetchall_arrayref({});
    $self->disconnect_db;

    my $urls = $self->new_instance_of("Urls");
    if ($rs) {
        for my $item (@$rs) {
            $self->_generalize($item);
            # check if feed?
            if ($item->{type} eq "feed") {
                my $url = $urls->select(id => $item->{srcid});
                if ($url) {
                    $item->{url} = $url->{url};
                }
            }
        }
    }

    $self->_select_tags($rs);
    return $rs->[0] if ($rs && $unique);
    return $rs;
}

# select user's timelime
sub _select_user {
    my ($self, $uid, $params) = @_;
    my ($limit_clause, $limit_values) = $self->build_limit_clause(params => $params);

    # show future story?
    my $extra_where_clause = "";
    if (!$params->{show_future}) {
        $extra_where_clause = $extra_where_clause . " AND firehose.createtime <= NOW() ";
    }

    # show non-public story?
    if (!$params->{show_nonpublic} && !$params->{public}) {
        $extra_where_clause = $extra_where_clause . " AND firehose.public != 'no' ";
    }

    my $sql = <<"EOSQL";
SELECT firehose.*,
       firehose_text.*,
       topics.*,
       users.nickname AS author
  FROM firehose
  JOIN firehose_text ON firehose.id = firehose_text.id
  JOIN users ON firehose.uid = users.uid
  JOIN topics ON firehose.tid = topics.tid
  WHERE firehose.uid = ?
  $extra_where_clause
  ORDER BY createtime DESC $limit_clause
EOSQL
    my @params;
    push @params, $uid;
    push @params, @$limit_values;

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute(@params);
    my $rs = $sth->fetchall_arrayref({});
    $self->disconnect_db;

    if ($rs) {
        for my $item (@$rs) {
            $self->_generalize($item);
        }
    }
    $self->_select_tags($rs);
    return $rs;
}

# select user's friends' timelime
sub _select_friends {
    my ($self, $uid, $params) = @_;
    my ($limit_clause, $limit_values) = $self->build_limit_clause(params => $params);

    my $sql = <<"EOSQL";
SELECT firehose.*,
       firehose_text.*,
       topics.*,
       users.nickname AS author
  FROM firehose
  JOIN firehose_text ON firehose.id = firehose_text.id
  JOIN users ON firehose.uid = users.uid
  JOIN topics ON firehose.tid = topics.tid
  WHERE firehose.uid IN (SELECT person FROM people
                           WHERE uid = ? AND type = "friend")
  ORDER BY firehose.createtime DESC $limit_clause;
EOSQL

    my @params;
    push @params, $uid;
    push @params, @$limit_values;

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute(@params);
    my $rs = $sth->fetchall_arrayref({});
    $self->disconnect_db;

    if ($rs) {
        for my $item (@$rs) {
            $self->_generalize($item);
        }
    }
    $self->_select_tags($rs);
    return $rs;
}

sub _generalize {
    my ($self, $item) = @_;

    # NTO-nized
    $item->{id} = $item->{srcid};
    $item->{create_time} = $item->{createtime};
    $item->{update_time} = $item->{last_update};

    my $topic = {};
    for my $k (qw(tid keyword textname series
                  image witdh height submittable
                  searchable storypickable usersprite)) {
        $topic->{$k} = $item->{$k};
        delete $item->{$k};
    }
    $item->{topics} = [$topic];
    $item->{primary_topic} = $topic;

    $item->{content_type} = $item->{type};
    $item->{intro_text} = $item->{introtext};
    $item->{body_text} = $item->{bodytext};
    $item->{discussion_id} = $item->{discussion};

    if ($item->{type} eq 'comment') {
        $item->{cid} = $item->{srcid};
        $item->{comment_id} = $item->{srcid};
    }
    elsif ($item->{type} eq 'story') {
        $item->{stoid} = $item->{srcid};
        $item->{story_id} = $item->{srcid};
    }
    elsif ($item->{type} eq 'journal') {
        $item->{journal_id} = $item->{srcid};
    }
    elsif ($item->{type} eq 'submission') {
        $item->{subid} = $item->{srcid};
        $item->{submission_id} = $item->{srcid};
    }

    # no public flag given, public is 'yes'
    $item->{public} = 'yes' if !$item->{public};

    #my $max_weight = 0;
    #for my $t (@{$item->{topics}}) {
    #    if ($t->{tid} == $item->{tid}) {
    #        $item->{primary_topic} = $t;
     #   }
        #if ($t->{weight} && $t->{weight} > $max_weight) {
        #    $max_weight = $t->{weight};
        #}
    #}
}


1;
