package Newslash::Model::Users::ACL2;
use Newslash::Model::Base -base;
use Data::Dumper;

sub al2_types {
    my $self = shift;
    my $types = $self->generic_select(table => "al2_types",
                                      keys => [qw(bitpos)],
                                      params => {
                                                 order_by => { bitpos => 'DESC' },
                                                });
    return $types;
}


sub types {
    my $self = shift;
    my $types = $self->al2_types;
    return @$types if (wantarray);
    my $rs = {};
    for my $type (@$types) {
        $rs->{$type->{name}} = $type;
        $rs->{$type->{al2tid}} = $type;
    }
    return $rs;
}

sub select {
    my ($self, $key, $value) = @_;
    my $sql;

    if ($key eq 'uid') {
        $sql = <<"EOSQL";
SELECT * FROM al2
  WHERE srcid = ?
EOSQL
    }
    elsif ($key eq 'srcid' || $key eq 'src_id') {
        $sql = <<"EOSQL";
SELECT * FROM al2
  WHERE srcid = CAST(CONV(?, 16, 10) AS UNSIGNED)
EOSQL
    }
    else {
        return;
    }

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute($value);
    my $rs = $sth->fetchall_arrayref(+{});
    $self->disconnect_db();

    return if !$rs;
    return {srcid => $value, value => 0, values => [], updatecount => 0} if @$rs == 0;

    my @values;

    my $val = $rs->[0]->{value};
    my $types = $self->al2_types;
    #my @sorted_types = sort { ($b->{bitpos} + 0) <=> ($a->{bitpos} + 0) } @$types;

    for my $type (@$types) {
        next if (!defined $type->{bitpos});
        my $number = 1 << $type->{bitpos};
        if ($val >= $number) {
            $val -= $number;
            push @values, $type;
        }
    }
    my $ret_val = $rs->[0];
    $ret_val->{values} = \@values;
    return $ret_val;
}

sub select_log {
    my ($self, $key, $value) = @_;
    my $sql;

    if ($key eq 'uid') {
        $sql = <<"EOSQL";
SELECT al2_log.*, al2_log_comments.comment, al2_types.title, users.nickname FROM al2_log
  JOIN al2_types USING(al2tid)
  JOIN al2_log_comments USING(al2lid)
  JOIN users ON users.uid = al2_log.adminuid
  WHERE al2_log.srcid = ?
EOSQL
    } elsif ($key eq 'srcid' || $key eq 'src_id') {
        $sql = <<"EOSQL";
SELECT al2_log.*, al2_log_comments.comment, al2_types.title, users.nickname FROM al2_log
  JOIN al2_types USING(al2tid)
  JOIN al2_log_comments USING(al2lid)
  JOIN users ON users.uid = al2_log.adminuid
  WHERE al2_log.srcid = CAST(CONV(?, 16, 10) AS UNSIGNED)
EOSQL

    }
    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute($value);
    my $rs = $sth->fetchall_arrayref(+{});
    $self->disconnect_db();

    return $rs;
}

sub update {
    #my ($self, $type, $srcid, $admin_uid, $types, $comment) = @_;
    my $self = shift;
    my $params = {@_};
    my $uid = $params->{uid};
    my $srcid = $params->{srcid} || $params->{src_id};
    my $admin_uid = $params->{admin_uid};
    my $types = $params->{types};
    my $comment = $params->{comment};

    my $acl2_types = $self->types;
    my $acl2_tids = [];
    my $value = 0;

    my $srcid_sql;
    my $current_acl2;
    if (defined $srcid) {
        $srcid_sql = "CAST(CONV(?, 16, 10) AS UNSIGNED)";
        $current_acl2 = $self->select(srcid => $srcid);
    }
    elsif (defined $uid) {
        $srcid = $uid;
        $srcid_sql = "?";
        $current_acl2 = $self->select(uid => $srcid);
    }
    else {
        return;
    }

    my $need_clear = {};
    if ($current_acl2) {
        for my $val (@{$current_acl2->{values}}) {
            $need_clear->{$val->{al2tid}} = 1;
        }
    }

    my $dbh = $self->start_transaction;
    my $sql = <<"EOSQL";
INSERT INTO al2_log
    (srcid,      ts,    adminuid, al2tid, val)
  VALUES
    ($srcid_sql, NOW(), ?,        ?,      "set")
EOSQL
    my $sth = $dbh->prepare($sql);

    my $rs;
    my @updated_ids;
    # insert al2_log
    for my $type (@$types) {
        my $type = $acl2_types->{$type};
        next if !$type;
        $rs = $sth->execute($srcid, $admin_uid, $type->{al2tid});
        if (!defined $rs) {
            $self->set_error($dbh->errstr, $dbh->err);
            $self->rollback;
            return;
        }
        if ($type->{bitpos} ne 'NULL') {
            $value += 1 << $type->{bitpos};
        }
        if ($need_clear->{$type->{al2tid}}) {
            $need_clear->{$type->{al2tid}} = 0;
        }
        my $al2lid = $dbh->last_insert_id(undef, undef, undef, undef);
        push @updated_ids, $al2lid;
    }

    # clear al2_log
    $sql = <<"EOSQL";
INSERT INTO al2_log
    (srcid,      ts,    adminuid, al2tid, val)
  VALUES
    ($srcid_sql,   NOW(), ?,        ?,      "clear")
EOSQL
    $sth = $dbh->prepare($sql);
    for my $tid (keys %$need_clear) {
        $rs = $sth->execute($srcid, $admin_uid, $tid);
        if (!defined $rs) {
            $self->set_error($dbh->errstr, $dbh->err);
            $self->rollback;
            return;
        }
        my $al2lid = $dbh->last_insert_id(undef, undef, undef, undef);
        push @updated_ids, $al2lid;
    }

    # insert al2_log_comments
    $sql = <<"EOSQL";
INSERT INTO al2_log_comments
    (al2lid, comment)
  VALUES
    (?,      ?)
EOSQL
    $sth = $dbh->prepare($sql);
    for my $al2lid (@updated_ids) {
        $rs = $sth->execute($al2lid, $comment);
        if (!defined $rs) {
            $self->set_error($dbh->errstr, $dbh->err);
            $self->rollback;
            return;
        }
    }

    # update al2
    $sql = <<"EOSQL";
INSERT INTO al2
    (srcid,      value, updatecount) 
  VALUES
    ($srcid_sql, ?,     0)
  ON DUPLICATE KEY UPDATE 
    value = ?,
    updatecount = updatecount + 1
EOSQL
    $sth = $dbh->prepare($sql);
    $rs = $sth->execute($srcid, $value, $value);
    if (!defined $rs) {
        $self->set_error($dbh->errstr, $dbh->err);
        $self->rollback;
        return;
    }

    # finished
    $self->commit;
    return 1;
}

sub hard_delete {
    my $self = shift;
    my $params = {@_};

    my $dbh = $self->start_transaction;

    my $srcid = $params->{srcid} || $params->{src_id};
    my $sql_where;
    my $update_count;

    if ($params->{uid}) {
        $sql_where = "WHERE srcid = ?";
        $srcid = $params->{uid};
    }
    else {
        $sql_where = "WHERE srcid = CAST(CONV(?, 16, 10) AS UNSIGNED)";
    }

    if ($srcid) {
        my $sql = "DELETE FROM al2 $sql_where";
        my $rs = $dbh->do($sql, undef, $srcid);
        if (!defined $rs) {
            $self->rollback;
            warn "DELETE al2 failed!";
            return;
        }
        $update_count = $rs;

        my $logs;
        if ($params->{uid}) {
            $logs = $self->select_log(uid => $params->{uid});
        }
        else {
            $logs = $self->select_log(src_id => $srcid);
        }
        for my $log (@$logs) {
            $rs = $dbh->do("DELETE FROM al2_log WHERE al2lid = ?", undef, $log->{al2lid});
            $update_count += $rs;
            $rs = $dbh->do("DELETE FROM al2_log_comments WHERE al2lid = ?", undef, $log->{al2lid});
            $update_count += $rs;
        }
    }

    $self->commit;
    return $update_count;

}

1;
