package Newslash::Model::Users::Class;
use Newslash::Model::Base -base;

sub key_definition {
    return {
            table => "users_class",
            primary => "uid",
            other => [qw(rookie)],
            datetime => [qw(rookie_cleared validated_submitter
                         validated_commenter validated_moderator story_passive
                         story_casual story_rater story_creator story_hardcore
                         comment_passive comment_casual comment_rater
                         comment_creator comment_hardcore)],
            aliases => { user_id => "uid",
                         id => "uid",
                         create_time => "created_at",
                       }
           };
}


sub select {
    my $self = shift;
    my $params = {@_};
    return $self->generic_select(params => $params);
}

sub update {
    my $self = shift;
    my $params = {@_};
    return $self->generic_update(params => $params);
}

=head2 clear_rookie($uid, uid => $uid, time => $time)

clear Rookie flag.

=over 4

=item Return value

1/0

=back

=cut
sub clear_rookie {
    my $self = shift;
    return if $self->check_readonly;
    my $params = {@_};
    my $uid = $params->{uid};
    return if !$uid;

    my $time = $params->{time};
    if ($time) {
        return $self->update(uid => $uid,
                             rookie => 0,
                             rookie_cleared => $time);
    }
    else {
        return $self->update(uid => $uid,
                             rookie => 0,
                             rookie_cleared => { function => "NOW()" });
    }
    return;
}

sub _validate {
    my $self = shift;
    my $params = {@_};
    return if !$params->{uid} || !$params->{target};
    my $users = $self->new_instance_of("Users");
    return if !$users->is_anonymous_uid($params->{uid});

    if ($params->{time}) {
        return $self->$self->update(uid => $params->{uid},
                                    $params->{target} => $params->{time}
                                   );
    }
    else {
        return $self->update(uid => $params->{uid},
                                    $params->{target} => { function => 'NOW()'}
                                   );
    }
}

1;
