package Newslash::Model::Users::Sidebar;
use Newslash::Model::Base -base;

sub key_definition {
    return {
            table => "ns_users_sidebar",
            primary => "uid",
            other => "items",
            aliases => { user_id => "uid",
                         id => "uid",
                         create_time => "created_at",
                       }
           };
}

sub on_start_up {
    my $self = shift;

    # create tables
    return 1 if $self->check_readonly;

    my $table_name = "ns_users_sidebar";
    if (!$self->table_exists($table_name)) {
        my $dbh = $self->connect_db;

        my $sql = <<"EOSQL";
CREATE TABLE IF NOT EXISTS $table_name (
  uid   mediumint(8) unsigned NOT NULL UNIQUE,
  items varchar(256)          DEFAULT ""
) DEFAULT CHARSET=utf8mb4
EOSQL
        my $rs = $dbh->do($sql, undef);
        $self->disconnect_db;
        if ($rs) {
            $self->warn("create table: $table_name");
        }
        else {
            $self->warn("cannot create table: $table_name");
        }
    }

    return 1;
}


sub select {
    my $self = shift;
    my $params = {@_};
    my $rs = $self->generic_select(params => $params);
    return [] if !$rs;

    my @items = split /\s+/, $rs->{items};
    return \@items;
}

sub update {
    my $self = shift;
    return if $self->check_readonly;
    my $params = {@_};
    my $uid = $params->{uid};
    return if !$uid;
    my $value = $params->{config} || {};

    # check params
    return if ref($value) ne "ARRAY";
    for my $k (@$value) {
        return if ($k !~ /^\d+$/);
    }
    my $val = join(" ", @$value);

    my $table_name = "ns_users_sidebar";
    my $sql = <<"EOSQL";
INSERT $table_name (uid, items) VALUES (?, ?)
  ON DUPLICATE KEY UPDATE items = ?
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->do($sql, undef, $uid, $val, $val);
    if (!defined $rs) {
        $self->disconnect_db;
        return;
    }

    $self->disconnect_db;
    return $value;
}

sub get_sidebar_contents {
    my $self = shift;
    my $params = {@_};

    my $uid = $params->{uid} || $params->{user} && $params->{user}->{uid};
    return if !$uid;

    my $boxes = $self->new_instance_of("Newslash::Model::Boxes");
    my $users = $self->new_instance_of("Users");
    my $user_sidebar = $self->select(uid => $uid);

    if (!$user_sidebar || @$user_sidebar == 0) {
        $user_sidebar = $self->select(uid => $users->anonymous_user->{uid});
    }

    # fallback
    if (@$user_sidebar == 0) {
        $user_sidebar =  [1, 2];
    }

    my $user_boxes = [];
    for my $box_id (@$user_sidebar) {
        my $box = $boxes->select(id => $box_id);
        if (!$box) {
            $box = {template => "error: invalid sidebar item; ID is $box_id"};
        }
        push @$user_boxes, $box;
    }
    return $user_boxes;
}


1;
