package Newslash::Model::Users::Util;

use Digest::MD5 qw(md5_hex md5_base64);

use strict;
use warnings;
use utf8;
use feature ':5.10';

use Exporter 'import';
our @EXPORT_OK = qw(nickname_to_matchname
                    generate_random_password
                    encrypt_password
                  );


#========================================================================

=head2 nickname_to_matchname

Convert nickname to matchname.

nickname consists of almost all alphabet, number, symbol.

matchname consists of lower alphabet and number.

=cut

sub nickname_to_matchname {
    my $nick = shift;
    $nick = lc $nick;
    $nick =~ s/[^a-zA-Z0-9]//g;
    return $nick;
}

#========================================================================

=head2 random_password

(import from: changePassword)

Return new random 8-character password composed of 0..9, A..Z, a..z
(but not including possibly hard-to-read characters [0O1Iil]).

=over 4

=item Return value

Random password.

=back

=cut

sub generate_random_password {
    my @chars = grep !/[0O1Iil]/, 0..9, 'A'..'Z', 'a'..'z';
    return join '', map { $chars[rand @chars] } 0 .. 7;
}

#========================================================================

=head2 encrypt_password

(import from: encryptPassword)

Encrypts given password, using the most recent salt (if any) in
Slash::Apache::User::PasswordSalt for the current virtual user.
Currently uses MD5, but could change in the future, so do not
depend on the implementation.

=over 4

=item Parameters

=over 4

=item PASSWD

Password to be encrypted.

=back

=item Return value

Encrypted password.

=back

=cut

sub encrypt_password {
    my ($passwd, $uid, $salt) = @_;
    $uid ||= '';
    $salt ||= '';

    #my $slashdb = getCurrentDB();
    #my $vu = $slashdb->{virtual_user};
    #my $salt = Slash::Apache::User::PasswordSalt::getCurrentPwSalt($vu);
    #$passwd = Encode::encode_utf8($passwd) if getCurrentStatic('utf8');

    #my $salt = $self->_get_salt;
    if ($uid) {
        return md5_hex("$salt:$uid:$passwd");
    } else {
        return md5_hex("$salt$passwd");
    }
}

1;
