package Newslash::Model::WebMessages;
use Newslash::Model::Messages -base;
use Data::Dumper;

sub on_start_up {
    my $self = shift;
    my $app = shift;

    return 1;
}

sub count_unread {
    my ($self, $uid) = @_;
    return if !$uid;

    my $sql = <<"EOSQL";
SELECT COUNT(id) FROM message_web
  WHERE user = ? AND readed = 0
EOSQL

    my $dbh = $self->connect_db;
    my $rs = $dbh->selectrow_arrayref($sql, undef, $uid);
    $self->disconnect_db;

    return $rs->[0] || 0;
}

sub select {
    my $self = shift;
    my $params = {@_};
    my @q_args;
    my $where_clause;

    my $uid = $params->{user} || $params->{uid};
    my $from_uid = $params->{fuser} || $params->{from_uid};
    my $id = $params->{id};
    my $keys = [qw(id user fuser code updated readed date)];
    my $unique = 0;

    if ($uid) {
        $where_clause = "message_web.user = ?";
        push @q_args, $uid;
    }
    elsif ($from_uid) {
        $where_clause = "message_web.fuser = ?";
        push @q_args, $from_uid;
    }
    elsif ($id) {
        $where_clause = "message_web.id = ?";
        push @q_args, $id;
        $unique = 1;
    }
    if ($where_clause) {
        $where_clause = "WHERE $where_clause";
    };

    my ($limit_clause, $values) = $self->build_limit_clause(params => $params);
    if ($limit_clause) {
        push @q_args, @$values;
    }

    my ($order_by_clause, $values) = $self->build_order_by_clause(keys => $keys,
                                                                 params => $params);

    my $sql = <<"EOSQL";
SELECT message_web.*,
       message_web_text.*,
       to_user.nickname AS to_nickname,
       from_user.nickname AS from_nickname
  FROM message_web
  JOIN message_web_text ON message_web.id = message_web_text.id
  JOIN users AS to_user ON message_web.user = to_user.uid
  LEFT JOIN users AS from_user ON message_web.fuser = from_user.uid
  $where_clause
  $limit_clause
EOSQL

    my $dbh = $self->connect_db;
    my $sth = $dbh->prepare($sql);
    $sth->execute(@q_args);
    my $rs = $sth->fetchall_arrayref({});
    $self->disconnect_db;

    return if !defined $rs;
    if ($unique) {
        return $rs->[0];
    }
    return $rs;
}

sub create {
    my ($self, $id, $uid, $from_uid, $title, $body, $code, $options) = @_;
    return if $self->check_readonly;
    return if (!$uid || !$title || !$body || !$code);
    $options ||= {};

    my $sql = <<"EOSQL";
INSERT INTO message_web
    (id, user, fuser, code, date)
  VALUE
    (?,  ?,    ?,     ?,    NOW())
EOSQL

    my $dbh = $self->start_transaction;
    my $sth = $dbh->prepare($sql);
    my $rs = $sth->execute($id, $uid, $from_uid, $code);
    if (!$rs) {
        $self->rollback;
        return;
    }

    $sql = <<"EOSQL";
INSERT INTO message_web_text
    (id, subject, message)
  VALUE
    (?,  ?,       ?)
EOSQL
    $sth = $dbh->prepare($sql);
    $rs = $sth->execute($id, $title, $body);
    if (!$rs) {
        $self->rollback;
        return;
    }
    $self->commit;
    return $id;
}

sub hard_delete {
    my $self = shift;
    my $params = {@_};

    my $id = $params->{id};
    return if !$id;

    my $sql = <<"EOSQL";
DELETE FROM message_web WHERE id = ?
EOSQL

    my $dbh = $self->start_transaction;
    my $rs = $dbh->do($sql, undef, $id);
    if (!$rs) {
        $self->rollback;
        return;
    }

    $sql = <<"EOSQL";
DELETE FROM message_web_text WHERE id = ?
EOSQL
    $rs = $dbh->do($sql, undef, $id);
    if (!$rs) {
        $self->rollback;
        return;
    }

    $self->commit;
    return 1;
}

1;
