package Newslash::Plugin::Hsts;
use Mojo::Base 'Mojolicious::Plugin';
use Mojo::URL;

use constant MAX_AGE_DEFAULT => 7776000;

sub register {
    my ($self, $app, $conf) = @_;
    $conf ||= {};

    # set default config value
    my $cnf = $app->config->{Hsts} ||= {};
    $cnf->{redirect_non_ssl} = 0 if !defined $cnf->{redirect_non_ssl};

    my $opts = { %$conf, %$cnf };
    my $max_age = defined $opts->{max_age} ? $opts->{max_age} : MAX_AGE_DEFAULT;
    my $redirect_non_ssl = $opts->{redirect_non_ssl};

    $app->hook(before_dispatch => sub {
                   my $c = shift;
                   return if $c->tx->is_websocket;

                   if ($c->req->is_secure) {
                       $c->res->headers->add('Strict-Transport-Security' => "max-age=$max_age");
                       return;
                   }
                   else {
                       return if !$redirect_non_ssl;
                       my $url = $c->tx->req->url->to_abs;
                       $url->scheme('https');
                       $c->res->code(301); # permanently redirect
                       $c->redirect_to($url);
                       return;
                   }
               });
}



1;
