package Newslash::Plugin::TimeLimitedCache;
use Mojo::Base 'Mojolicious::Plugin';
use Data::Dumper;
use Storable qw(dclone);

sub _make_clone {
    my ($self, $val) = @_;
    if (ref($val)) {
        return dclone($val);
    }
    return $val;
}

sub get {
    my ($self, $key) = @_;
    my $data = $self->{_cache}->get($key);

    return if !defined $data;

    if ($self->{test_mode}) {
        $self->_del($key);
        return;
    }

    if (!defined $data->{expire}) {
        $self->_del($key);
        return;
    }

    if ($data->{expire} == 0) {
        return $self->_make_clone($data->{value});
    }

    if ($data->{expire} && ($data->{expire} > time)) {
        return $self->_make_clone($data->{value});
    }

    $self->_del($key);
    return;
}

sub set {
    my ($self, $key, $value, $expire) = @_;
    my $expire_time = $expire ? (time + $expire) : 0;
    my $data = {value => $self->_make_clone($value),
                expire => $expire_time};

    $self->{_cache}->set($key, $data);
}

sub del {
    my ($self, $key) = @_;
    my $val = $self->get($key);
    if ($val) {
        $self->_del($key);
        return $val;
    }
    return;
}

sub _del {
    my ($self, $key) = @_;
    $self->{_cache}->set($key, undef);
    return;
}

sub register {
    my ($self, $app, $conf) = @_;

    my $cnf = $app->config->{MemoryCache} ||= {};
    $cnf->{test_mode} = 0 if !defined $cnf->{test_mode};
    $cnf->{max_keys} = 1000 if !defined $cnf->{max_keys};

    $self->{conf} = {%$conf, %$cnf};
    if ($self->{conf}->{test_mode}) {
        # if test_mode == 1, already yield cache miss
        $self->{test_mode} = 1;
    }

    $self->{app} = $app;
    $self->{max_keys} = $self->{conf}->{max_keys} || 1000;
    $self->{_cache} = Mojo::Cache->new(max_keys => $self->{max_keys});
    $app->helper(cache => sub {state $cache = $self});
}

1;

