package Newslash::Util::Timezones;

use strict;
use warnings;
use utf8;
use feature ':5.10';

use Exporter 'import';

our @EXPORT_OK = qw(
                       get_timezone_offset
                       get_supported_timezones
                  );


use constant TZCODE_HASH => {
                            ACST => [34200, "Central Australian", "Australia", "ACDT", 37800],
                            AEST => [36000, "Eastern Australian", "Australia", "AEDT", 39600],
                            AKST => [-32400, "Alaska", "America", "AKDT", -28800],
                            AST  => [-14400, "Atlantic", "America", "ADT", -10800],
                            AT   => [-7200, "Azores", undef, undef, undef],
                            AWST => [28800, "Western Australian", "Australia", "AWDT", 32400],
                            BT   => [10800, "Baghdad",  "USSR Zone 2", undef, undef, undef],
                            CCT  => [28800, "China Coast",  "USSR Zone 7", undef, undef, undef],
                            CET  => [3600, "Central European", "Europe", "CEST", 7200],
                            CST  => [-21600, "Central", "America", "CDT", -18000],
                            EET  => [7200, "Eastern European", "Europe", "EEST", 10800],
                            EST  => [-18000, "Eastern", "America", "EDT", -14400],
                            GMT  => [0, "Greenwich Mean", undef, undef, undef],
                            GST  => [-10800, "Greenland", undef, undef, undef],
                            HAST => [-36000, "Hawaii-Aleutian", "America", "HADT", -32400],
                            IDLE => [43200, "International Date Line East", undef, undef, undef],
                            IDLW => [-43200, "International Date Line West", undef, undef, undef],
                            IST  => [19800, "Indian", undef, undef, undef],
                            IT   => [12600, "Iran", undef, undef, undef],
                            JST  => [32400, "Japan, USSR Zone 8", undef, undef, undef],
                            JT   => [27000, "Java", undef, undef, undef],
                            KST  => [32400, "Korean", undef, "KDT", 36000],
                            MAGS => [39600, "Magadan", undef, "MAGD", 43200],
                            MST  => [-25200, "Mountain", "America", "MDT", -21600],
                            NST  => [-12600, "Newfoundland", "America", "NDT", -9000],
                            NT   => [-39600, "Nome", undef, undef, undef],
                            NZST => [43200, "New Zealand", "New Zealand", "NZDT", 46800],
                            PST  => [-28800, "Pacific", "America", "PDT", -25200],
                            UTC  => [0, "Universal Coordinated", undef, undef, undef],
                            WAT  => [-3600, "West Africa", undef, undef, undef],
                            WET  => [0, "Western European", "Europe", "WEST", 3600],
                            ZP4  => [14400, "USSR Zone 3", undef, undef, undef],
                            ZP5  => [18000, "USSR Zone 4", undef, undef, undef],
                            ZP6  => [21600, "USSR Zone 5", undef, undef, undef],
                            ZP7  => [25200, "USSR Zone 6", undef, undef, undef],
                           };


sub get_timezone_offset {
    my $tzcode = shift;
    my $use_dst = shift;

    my $codes = TZCODE_HASH->{$tzcode} || TZCODE_HASH->{UTC};
    if ($use_dst) {
        return $codes->[4];
    }
    return $codes->[0];
}

sub get_supported_timezones {
    return TZCODE_HASH();
}
