package Newslash::Web::Controller::API::Admin::Blocking;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

sub get {
    my $c = shift;
    my $param = $c->req->params->to_hash;

    my $limit = $param->{limit} || 20;
    my $skip = $param->{skip} || 0;
    my $type = $param->{type};

    if (!$type) {
        $c->render(json => {error => 1, message => "no_type_given"});
        $c->rendered(400);
        return;
    }

    my $model_name;
    my $url_prefix;
    my $id_key;

    if ($type eq 'story') {
        $model_name = 'stories';
        $url_prefix = "/story/";
        $id_key = "sid";
    }
    elsif ($type eq 'journal') {
        $model_name = 'journals';
        $url_prefix = "/journal/";
        $id_key = "journal_id";
    }
    elsif ($type eq 'submission') {
        $model_name = 'submissions';
        $url_prefix = "/submission/";
        $id_key = "subid";
    }
    else {
        $c->render(json => {error => 1, message => "invalid_type"});
        $c->rendered(400);
        return;
    }


    my $model = $c->app->model($model_name);
    my $items = $model->select(public => "no",
                               order_by => { "create_time" => "DESC" },
                               limit => $limit,
                               skip => $skip,
                              );
    if (!$items || ref($items) ne "ARRAY") {
        $c->rendered(500);
        return;
    }

    for my $item (@$items) {
        $item->{url} = $url_prefix . $item->{$id_key} . "/";
    }

    $c->render(json => {type => $type, items => $items});
    return;
}

sub post {
    my $c = shift;
    my $data = $c->req->json;
    my $action = $data->{action};
    my $type = $data->{type};
    my $id = $data->{id};

    my $message = "";

    $message = "no_action_given" if !$action;
    $message = "no_type_given" if !$type;
    $message = "no_id_given" if !$id;

    if ($message) {
        $c->render(json => {error => 1, message => "no_action_given"});
        $c->rendered(400);
        return;
    }

    if ($action eq "block") {
        return $c->_change_state($type, $id, "no");
    }
    elsif ($action eq "unblock") {
        return $c->_change_state($type, $id, "yes");
    }

    $c->render(json => {error => 1, message => "invalid_action"});
    $c->rendered(400);
    return;

}

sub _change_state {
    my ($c, $type, $id, $public) = @_;

    my $model_name;
    if ($type eq 'story') {
        $model_name = 'stories';
    }
    elsif ($type eq 'journal') {
        $model_name = 'journals';
    }
    elsif ($type eq 'submission') {
        $model_name = 'submissions';
    }
    else {
        $c->render(json => {error => 1, message => "invalid_type"});
        $c->rendered(400);
        return;
    }

    my $rs = $c->app->model($model_name)->update(id => $id,
                                                 public => $public);
    if (!$rs) {
        $c->render(json => {error => 1, message => "update_failed"});
        $c->rendered(500);
        return;
    }
    $c->render(json => { type => $type, id => $id, result => $rs });
    $c->rendered(200);
    return 1;
}



1;

