package Newslash::Web::Controller::API::Admin::Feed;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);

sub get {
    my $c = shift;
    my $feeds = $c->app->model('feeds');
    my $data = $c->req->params->to_hash;
    my $feed_id = $data->{id} || $data->{feed_id};

    # feed_id is not given, return all items
    if (!$feed_id) {
        my $items = $feeds->select(order_by => {create_time => "DESC"});
        if (!$items) {
            $c->rendered(500);
            return;
        }
        $c->render(json => {items => $items});
        return;
    }

    # return selected item
    my $item = $feeds->select(feed_id => $feed_id);
    if (!$item) {
        $c->rendered(404);
        return;
    }

    $c->render(json => {item => $item});
    return;
}

sub post {
    my $c = shift;
    my $data = $c->req->json;
    my $action = $data->{action} || "";

    if ($action eq "fetch") {
        return $c->_fetch;
    }
    elsif ($action eq "create") {
        return $c->_create;
    }
    elsif ($action eq "update") {
        return $c->_update;
    }
    elsif ($action eq "delete") {
        return $c->_delete;
    }

    $c->render(json => {error => 1, message => "no_action_given"});
    $c->rendered(400);
    return;
}

sub _fetch {
    my $c = shift;
    my $feeds = $c->app->model('feeds');
    my $data = $c->req->json;
    my $url = $data->{url};

    if (!$url) {
        $c->rendered(400);
        return;
    }

    my $feed = $feeds->fetch_url(\$url, $c->app->config->{FeedFetcher} || {});
    if (!$feed) {
        $c->render(json => {error => 1, message => $feeds->last_error});
        $c->rendered(400);
        return;
    }

    my $result = { url => $url,
                   title => $feed->title,
                   description => $feed->description};

    $c->render(json => {item => $result});
    return;
}

sub _create {
    my $c = shift;
    my $feeds = $c->app->model('feeds');
    my $data = $c->req->json;

    my $url = $data->{url};
    my $name = $data->{name};

    my $message = "";
    $message = "no_url_given" if !$url;
    $message = "no_name_given" if !$name;

    if ($message) {
        $c->render(json => {error => 1, message => $message});
        $c->rendered(400);
        return;
    }

    my $rs = $feeds->select(url => $url);
    if ($rs) {
        $c->render(json => {error => 1, message => "url_is_already_registered"});
        $c->rendered(400);
        return;
    }

    my $feed_id = $feeds->create(url => $url,
                                 name => $data->{name},
                                 title => $data->{title} || "",
                                 description => $data->{description} || "");
    if (!$feed_id) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {feed_id => $feed_id});
    return;
}

sub _update {
    my $c = shift;
    my $feeds = $c->app->model('feeds');
    my $data = $c->req->json;
    my $feed_id = $data->{id} || $data->{feed_id};

    if (!$feed_id) {
        $c->rendered(400);
        return;
    }

    my $rs = $feeds->select(feed_id => $feed_id);
    if (!$rs) {
        $c->render(json => {error => 1, message => "not_found"});
        $c->rendered(400);
        return;
    }

    $rs = $feeds->update(feed_id => $feed_id,
                         title => $data->{title},
                         description => $data->{description},
                         url => $data->{url},
                         status => $data->{status}
                        );
    if (!$rs) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {feed_id => $feed_id});
    return;
}

sub _delete {
    my $c = shift;
    my $feeds = $c->app->model('feeds');
    my $data = $c->req->json;
    my $feed_id = $data->{id} || $data->{feed_id};

    if (!$feed_id) {
        $c->rendered(400);
        return;
    }

    my $rs = $feeds->select(feed_id => $feed_id);
    if (!$rs) {
        $c->render(json => {error => 1, message => "not_found"});
        $c->rendered(400);
        return;
    }

    $rs = $feeds->delete(feed_id => $feed_id);

    if (!$rs) {
        $c->rendered(500);
        return;
    }

    $c->render(json => {error => 0});
    return;
}

1;

