package Newslash::Web::Controller::API::Journal;
use Mojo::Base 'Newslash::Web::Controller';
#use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use Mojo::JSON qw(decode_json encode_json);
use Newslash::Util::TextFormatter;

sub get {
    my $c = shift;
    my $params = $c->req->query_params->to_hash;
    my $user = $c->stash('user');
    my $journals = $c->model('journals');
    my $limit = 10;
    my $offset = $params->{offset} || 0;

    my $type = $params->{type} || 'latest';
    my $target = $user;
    my $journal_id = $params->{id} || $params->{journal_id};

    if ($journal_id) {
        my $journal = $journals->select(journal_id => $journal_id);
        if ($journal) {
            $journals->generate_bodytext($journal);
            $c->render(json => { journal => $journal });
            return;
        }
        else {
            $c->render(json => { error => 1, message => "not_found" });
            $c->rendered(400);
            return;
        }
    }

    if ($params->{nickname}) {
        my $users = $c->model('users');
        $target = $users->select(nickname => $params->{nickname});
    }

    if (!$target) {
        $c->render(json => { error => 1, message => "no valid user" });
        return;
    }

    if ($type eq 'latest') {
        my $journals = $c->model('journals')->select(uid => $target->{uid}, limit => $limit, offset => $offset);
        if ($journals) {
            $c->model('journals')->generate_bodytext($journals);
            $c->render(json => {user => $target, journals => $journals});
            return;
        }
        $c->render(json => {user => $target, journals => []});
        return;
    }
    $c->rendered(400);
    return;
}

sub post {
    my $c = shift;
    my $params = {};

    my $user = $c->stash('user');

    my $journals = $c->model('journals');
    my $users = $c->model('users');

    my $data = $c->req->json;
    my $item = $data->{item};
    my $message = "";

    $params->{description} = $c->format_htmltext($item->{title}, "title");
    $message = "no title!" if !$params->{description};

    # check user is valid
    if (!defined $user || !$user->{is_login}) {
        $message = "invalid author";
    }
    $params->{uid} = $user->{uid};
    $params->{article} = $item->{introtext} || $item->{intro_text};
    $params->{commentstatus} = $item->{comment_status} || $item->{comment_status};
    $params->{action} = $data->{action} || 'preview';
    $params->{posttype} = $item->{posttype} || $item->{post_type};
    $params->{tid} = $item->{tid};
    $params->{id} = $item->{id} if $item->{id};
    $params->{formatter} = $item->{formatter};

    if ($item->{submissioncopy}) {
        $params->{promotype} = "publicize";
    }
    #else {
    #    $params->{promotetype} = "publish";
    #}

    # TODO: can select topics
    #$params->{topics_chosen} = { 49 => 10, }; # tid => weight

    # parse tags
    #$params->{tid} = 49; #mainpage
    my $tags_string = $item->{tags_string};
    my $topic = {};
    my @tags;
    if ($tags_string) {
        @tags = split(/\s+/, $tags_string);
        $topic = $c->model('topics')->get_primary_topic_from_tags(\@tags);
        $params->{tags_string} = $tags_string;
    }
    $params->{tid} = $topic->{tid} || 49;

    #$extra_params->{createheaders} = $c->req->headers->to_string;


    if (length($message) > 0) {
        $c->render(json => { err => 1, message => $message });
        $c->rendered(400);
        return;
    }

    if ($data->{action} eq 'preview') {
        if ($item->{formatter} && $item->{formatter} eq 'legacy') {
            $item->{introtext} = $c->strip_by_mode($params->{article}, $params->{posttype});
        }
        else {
            $item->{introtext} = $c->format_htmltext($params->{article}, "journal");
        }
        $item->{intro_text} = $item->{introtext};
        $item->{title} = $params->{description};
        $item->{topic} = $topic;
        $c->render(json => {item => $item});
        return;
    }
    elsif ($params->{action} eq 'post') {
        my $journal_id;
        if ($params->{id}) {
            # update journal
            $journal_id = $journals->update(user => $user, %$params);
        }
        else {
            # create journal
            $journal_id = $journals->create(user => $user, %$params);
        }
        if ($journal_id) {
            $c->render(json => {type => "journal", id => $journal_id}, author => $user->{nickname});
            $c->event_que->emit("journal", "post", $user->{uid}, $journal_id);
            $c->stats->add_event_counter("journal_post");
        }
        else {
            if ($journals->last_errorno && $journals->last_errorno == 1062) { #ER_DUP_ENTRY
                $c->rendered(409);
                return;
            }
            $c->render(json => { error => 1, message => $journals->last_error });
            $c->rendered(500);
        }
    }
}

1;
