package Newslash::Web::Controller::Journal;
use Mojo::Base 'Mojolicious::Controller';
use Data::Dumper;
use List::Util qw(any);

sub single {
    my $c = shift;
    my $users = $c->model('users');
    my $user = $c->stash('user');
    my $journal_id = $c->stash('id');
    my $author_nickname = $c->stash('nickname');

    my $journals = $c->model('journals');
    my $journal = $journals->select(id => $journal_id);
    if (!$journal) {
        $c->render('Not found', status => '404');
        return;
    }

    my $the_user = $c->users->get_user(uid => $journal->{uid});

    # if no nickname given, redirect
    if (!$author_nickname || $author_nickname ne $the_user->{nickname}) {
        $c->res->code(301);
        $c->redirect_to("/~$the_user->{nickname}/journal/$journal_id/");
        return;
    }

    # generate bodytext content
    $journals->generate_bodytext($journal);

    # get discussion object
    my $d_id = $journal->{discussion};
    my $comments = [];
    my $discuss;
    if ($d_id) {
        my $discussions = $c->model('discussions');
        $discuss = $discussions->select(id => $d_id);
        $comments = $c->model('comments')->select(discussion_id => $d_id);
    }

    my $page = { content_type => "journal",
                 type => "single",
                 id => $journal->{journal_id},
                 journal_id => $journal->{journal_id},
               };

    $c->render(item => $journal,
               discussion => $discuss,
               comments => $comments,
               the_user => $the_user,
               page => $page,
               use_captcha => 1,
              );
    $c->stats->add_event_counter("journal_view");
}

sub create {
    my $c = shift;
    my $page = { content_type => "journal",
                 type => "single",
                 id => 0,
                 journal_id => 0,
               };

    $c->render(page => $page,
	       the_user => $c->stash('user'),
	);
}

sub user_journals {
    my $c = shift;
    my $users = $c->model('users');
    my $limit = 10;
    my $offset = 0;

    my $user_name = $c->stash('user_name');
    $user_name =~ s/\A~(.*)\z/$1/;
    my $user = $users->select(nickname => $user_name);

    my $journals = $c->model('journals')->select(uid => $user->{uid}, limit => $limit, offset => $offset);
    if (!$journals) {
        $c->render('Not found', status => '404');
        return;
    }

    $c->model('journals')->generate_bodytext($journals);
    $c->render(journals => $journals, journal_author => $user->{nickname});
}


1;

