package Newslash::Web::Controller::Poll;
use Mojo::Base 'Mojolicious::Controller';
use Mojo::Util qw(dumper);
use DateTime;
use DateTime::Format::MySQL;
use Data::Dumper;

sub _is_poll_active {
    return 1;
    my ($c, $poll) = @_;
    return 0 if $poll->{discussion_type} ne "open";

    my $polls_config = $c->app->config->{Polls} || {};

    if ($polls_config->{TTL} && 
        $polls_config->{TTL_unit} =~ m/\A(years|months|days|hours|minutes|seconds)\z/) {
        my $ttl_dt = DateTime::Format::MySQL->parse_datetime($poll->{date});
        $ttl_dt->add($polls_config->{TTL_unit} => $polls_config->{TTL});
        my $now = DateTime->now;
        if (DateTime->compare($ttl_dt, $now) == -1) {
            return 0;
        }
    }
    return 1;
}

sub single {
    my $c = shift;
    my $polls = $c->model('polls');
    my $user = $c->stash('user');
    my $qid = $c->stash('qid');

    my $poll = $polls->select(qid => $qid);
    if (!$poll) {
        $c->rendered(404);
        return;
    }
    my $discussions = $c->model('discussions');
    my $discussion_id = $poll->{discussion_id};
    my $discuss = $discussions->select(discussion_id => $discussion_id);
    my $comments = [];

    if ($discussion_id) {
        $comments = $c->model('comments')->select(discussion_id => $discussion_id);

    }

    $poll->{active} = _is_poll_active($c, $poll);

    my $related_polls = $c->model('polls')->select(limit => 10, order_by => {create_time => "DESC"});

    my $page = { content_type => "poll",
                 type => "single",
                 qid => $poll->{qid},
                 question_id => $poll->{qid},
                 id => $poll->{qid},
               };

    $c->render(item => $poll,
               page => $page,
               comments => $comments,
               use_captcha => 1,
               discussion => $discuss,
               related => $related_polls,
              );
}

sub vote {
    my $c = shift;
    my $polls = $c->model('polls');
    my $user = $c->stash('user');
    my $qid = $c->stash('qid');

    my $poll = $polls->select(qid => $qid);
    if (!$poll) {
        $c->rendered(404);
        return;
    }

    if (!_is_poll_active($c, $poll)) {
        $c->res->code(301);
        $c->redirect_to("/poll/$qid");
    }

    $poll->{active} = 1;
    $poll->{content_type} = "vote";

    $c->render(story => $poll,
              );
}

sub _is_valid_voter {
    my ($c, $user, $qid) = @_;
    my $polls = $c->model('polls');
    my $config = $c->app->config("Polls") || {};

    # check duplicate vote by uid/ipid
    my $voters;
    if ($user->{is_login}) {
        $voters = $polls->voters->select(question_id => $qid,
                                         user_id => $user->{uid},
                                         order_by => {time => "DESC"},
                                        );
    }
    elsif ($user->{ipid}) {
        $voters = $polls->voters->select(question_id => $qid,
                                         ip_id => $user->{ipid},
                                         limit => 1,
                                         order_by => {time => "DESC"},
                                        );
    }

    if (!$voters) {
        # error
        $c->app->log->warn("Poll: voter check failed! uid: $user->{uid}, ipid: $user->{ipid}");
        return 0;
    }

    if (@$voters > 0) {
        # already voted
        if ($user->{is_login}) {
            return 0;
        }

        # already voted from the same network. check timeout
        my $cooling_sec = $config->{vote_cooling_seconds} || 60;
        my $now = DateTime->now;
        my $cool_limit = DateTime::Format::MySQL->parse_datetime($voters->[0]->{time});
        $cool_limit->add(seconds => $cooling_sec);

        if (DateTime->compare($cool_limit, $now) > 0) {
            # now cooling time...
            return 0;
        }
    }

    return 1;
}

sub vote_post {
    my $c = shift;
    my $polls = $c->model('polls');
    my $user = $c->stash('user');
    my $qid = $c->stash('qid');
    my $aid = $c->req->param('answer');

    if ($qid !~ m/\A[0-9]+\z/) {
        $qid = 0;
    }
    if (!$qid) {
        $c->rendered(404);
        return;
    }

    my $poll = $polls->select(qid => $qid);
    if (!$poll) {
        $c->rendered(404);
        return;
    }

    if (!_is_poll_active($c, $poll)
        || !$c->_is_valid_voter($user, $qid)) {
        $c->res->code(303);
        $c->redirect_to("/poll/$qid");
        return;
    }

    my $rs = $polls->vote($user, $qid, $aid);
    if (!$rs) {
        $c->app->log->debug("vote failed. qid: $qid, aid: $aid, uid: $user->{uid}");
    }
    # require result check or not ?
    $c->res->code(303);
    $c->redirect_to("/poll/$qid");
}


1;
