package Newslash::Web::Controller::SiteConfig;
use Mojo::Base 'Newslash::Web::Controller';
use Data::Dumper;
use Encode qw(encode);

use constant CACHE_PREFIX => "siteconfig";
use constant DEFAULT_MEM_EXPIRE => 60 * 60;
use constant DEFAULT_KVS_EXPIRE => 60 * 60 * 24;

ub site_config {
    my $c = shift;
    my $epoch = $c->stash('epoch');

    # try to read from in-memory cache
    my $value = $c->ccache->get(CACHE_PREFIX, $epoch);

    if (!$value) {
        # generate siteconfig
        my $mod_reasons = $c->model('moderations')->reasons;
        my $topics = $c->model('topics')->select;
        my @topic_texts = map { $_->{textname} } @$topics;

        $value = $c->render_to_string("common/siteconfig",
                                      format => 'js',
                                      moderate_reasons => $mod_reasons,
                                      topics => \@topic_texts,
                                      acl2_types => {},
                                     );
        $c->set->cache(CACHE_PREFIX, $epoch, $value);
    }

    #$c->app->log->info($value);
    $c->res->headers->content_type('application/javascript');
    #$c->res->body($value->encode);
    $c->res->body(encode('UTF-8', $value));
    $c->rendered(200);
}


1;

