package Template::Plugin::MicroCalendar;
use strict;
use warnings FATAL => 'all';
our $VERSION = '1.00';

use Template::Plugin;
use Template::Iterator;
use Template::Exception;
use base qw( Template::Plugin );

use DateTime;

sub new {
    my ($class, $context, @args) = @_;
    return bless { _CONTEXT => $context,
                 }, $class;
}

sub day_of_week {
    my ($self, $year, $month, $day, $offset_sec) = @_;
    if (!$year || !$month || !$day) {
        die "MicroCalendar::day_of_week: invalid parameter";
    }
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => $day);
    $dt->subtract(seconds => $offset_sec || 0);
    return $dt->day_of_week;
}

sub last_day_of_month {
    my ($self, $year, $month) = @_;
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => 1);
    $dt->add(months => 1);
    $dt->add(days => -1);
    return $dt->day;
}

sub year_of_next_month {
    my ($self, $year, $month) = @_;
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => 1);
    $dt->add(months => 1);
    return $dt->year;
}

sub year_of_previous_month {
    my ($self, $year, $month) = @_;
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => 1);
    $dt->add(months => -1);
    return $dt->year;
}

sub next_month {
    my ($self, $year, $month) = @_;
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => 1);
    $dt->add(months => 1);
    return $dt->month;
}

sub previous_month {
    my ($self, $year, $month) = @_;
    my $dt = DateTime->new(year => $year,
                           month => $month,
                           day => 1);
    $dt->add(months => -1);
    return $dt->month;
}

1;
