/* blocking-manager.js */
const blockingManager = {};

blockingManager.run = function run (params) {
  var vm;
  const data = {
    message: '',
    items: [],
    targetUrl: '',
  };
  const computed = {};
  const methods = {
    unblockItem: unblockItem,
    blockUrl: blockUrl,
    reloadItems: reloadItems,
  };

  function unblockItem() {
    var type = "submission";
    this.items.forEach(item => {
      if (!item.selected) return;
      newslash.admin.unblockItem(type, item.id).then(
        (resp) => {
          item.deleted = 1;
        },
        (resp) => {
          if (resp) {
            this.message = "error: " + resp.message;
          }
        }
      );
    });
  }

  function blockUrl() {
    var type = "submission";
    var url = newslash.urlParse(this.targetUrl);
    var re = new RegExp("^/" + type + "/([0-9/]+)$");

    var result = re.exec(url.pathname);
    if (result && result[1]) {
      statusBar.loading("sending request...");
      newslash.admin.blockItem(type, result[1]).then(
        (resp) => {
          this.reloadItems(type);
          this.targetUrl = "";
          statusBar.hide();
        },
        (resp) => {
          if (resp) {
            this.message = "error: " + resp.message;
            statusBar.error(resp.message);
          }
        }
      );
    } else {
      this.message = "invalid URL";
    }    
  }

  function reloadItems (type) {
    newslash.admin.getBlockedItems(type).then(
      (resp) => { // success
        this.items = [];
        this.items = resp.items || [];
        message = resp.message || '';
        initItems(this.items);
      },
      (resp) => { // fail
        this.items = [];
        if (resp) {
          this.message = resp.message || "failed to get items";
        }
      }
    );
  }

  function initItems (items) {
    items.forEach(item => {
      Vue.set(item, 'selected', false);
      Vue.set(item, 'deleted', false);
    });
  };

  vm = new Vue({el: params.el,
                data: data,
                computed: computed,
                methods: methods,
                created: function () {
                  this.reloadItems("submission");
                },
               });
};
