var contentEditor = {};

contentEditor.run = function run(params) {
  const props = {
    item: {},
  };

  function data () {
    const editor = {
      introtext: "",
      bodytext: "",
      title: "",
      commentstatus: "enabled",
      submissioncopy: 0,
      url: "",
      email: "",

      previewIntro: "",
      previewBody: "",
      previewTitle: "",
      
      showForm: true,
      showSubmit: false,
      message: "",
      createdUrl: "",
      
      allowedTags: params.config.allowed_tags || {},
      urls: {
        submission: '/api/v1/submission',
        story: '/api/v1/story',
        journal: '/api/v1/journal',
      },
    };
      
    Vue.set(this.item, 'editor', this);
    return editor;
  };

  const watch = {
    introtext: function (val, oldVal) {
      this.item.introtext = (val.length > 0) ? this.quoteHtml(val) : "";
    },
    bodytext: function (val, oldVal) {
      this.item.bodytext = (val.length > 0) ? this.quoteHtml(val) : "";
    },
    title: function (val, oldVal) {
      this.item.title = (val.length > 0) ? this.quoteTitle(val) : "";
    },
    url: function (val, oldVal) {
      this.item.url = (val.length > 0) ? val : "";
    },
  };

  const methods = {
    showPreview: function (event) {
      const url = this.urls[this.item.content_type];
      const postData = {};
      Object.keys(this.item).forEach(k => {
        // remove circular reference
        if (k != 'editor') {
          postData[k] = this.item[k];
        }
      });
      this.$http.post(url, {item: postData, action: 'preview'}).then(
        (response) => { // success
          this.message = "";
          this.previewTitle = response.body.item.title;
          this.previewIntro = response.body.item.introtext;
          this.showSubmit = true;
          this.showForm = false;
        },
        (response) => { // fail
          if (response.body.message) {
            this.message = response.body.message;
          }
        }
      );
    },
    postSubmission: function (event) {
      this.message = "";
      const url = this.urls[this.item.content_type];
      const postData = {};
      Object.keys(this.item).forEach(k => {
        // remove circular reference
        if (k != 'editor') {
          postData[k] = this.item[k];
        }
      });
      this.$http.post(url, {item: postData, action: 'post'}).then(
        (response) => { // success
          this.message = "";
          const id = response.body.id;
          const type = response.body.type;
          const url = '/' + type + '/' + id;
          if (!postData.id) { // new post
            this.createdUrl = url;
            this.message = "post_success";
          }
          this.item.editing = false;
          this.showSubmit = false;
          this.showForm = true;
        },
        (response) => { // fail
          if (response.body.message) {
            this.message = response.body.message;
          }
        }
      );
    },
    leavePreview: function (event) {
      this.previewTitle = "";
      this.previewIntro = "";
      this.previewBody = "";
      this.showSubmit = false;
      this.showForm = true;
    },
    quoteHtml: function (html) {
      const escaped = eh.escape(this.allowedTags, html);
      const quoted = eh.blank_line_to_paragraph(escaped);
      const parsed = $.parseHTML(quoted, null);
      const result = [];
      parsed.forEach((i) => {
        i.normalize();
        result.push(i.outerHTML);
      });
      return result.join('\n');
    },
    quoteTitle: function (html) {
      const escaped = '<div>' + eh.escape({}, html) + '</div>';
      const parsed = $.parseHTML(escaped, null);
      const result = [];
      parsed.forEach((i) => {
        i.normalize();
        const t = i.innerHTML || i.wholeText;
        result.push(t);
      });
      return result.join('');
    },
    startEdit: function () {
      this.original = {
        introtext: this.item.introtext,
        title: this.item.title,
      };
      this.introtext = this.item.article;
      this.title = this.item.title;
    },
    cancelEdit: function () {
      this.item.introtext = this.original.introtext;
      this.item.title = this.original.title;
      this.item.editing = false;
    }
  };

  Vue.component('content-editor', {
    template: '#content-editor',
    props: props,
    data: data,
    watch: watch,
    methods: methods,
  });

};
