# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";


if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("boxtestadmin", $test_passwd);
    ok($admin, "create admin user");
}

subtest 'create/select/update/delete/exports boxes' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    # login
    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => $admin->{nickname}, passwd => $test_passwd})
      ->status_is(302);

    my $boxes = $t->app->model('boxes');

    # delete duplicated entry before
    for my $name (qw(test_sidebar_01 test_sidebar_02)) {
        my $box = $boxes->select(name => $name);
        if ($box) {
            my $rs = $boxes->delete(id => $box->{id});
        }
    }

    # create
    my $test_data = {
                     action => "create",
                     name => "test_sidebar_01",
                     model => 'stories',
                     query_params => 'foo',
                     template => 'foobar',
                    };

    $t->post_ok('/api/v1/admin/sidebar' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/id');

    my $id = $t->tx->res->json->{id};
    if (!$id) {
        diag(Dumper($t->tx->res->json));
        return;
    }

    # update
    $test_data = {
                  action => "update",
                  id => $id,
                  name => 'test_sidebar_02',
                  comment => 'hoge',
                  model => 'stories',
                  query_params => 'foo',
                  template => 'foobar',
                 };


    $t->post_ok('/api/v1/admin/sidebar' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/id');

    $id = $t->tx->res->json->{id};
    diag(Dumper($t->tx->res->json)) if !$id;

    $t->get_ok("/api/v1/admin/sidebar?id=$id")
      ->status_is(200)
      ->json_has('/item')
      ->json_has('/item/id')
      ->json_has('/item/name')
      ->json_has('/item/comment')
      ->json_has('/item/model')
      ->json_has('/item/query_params')
      ->json_has('/item/template');

    $t->get_ok("/api/v1/sidebar/item")
      ->status_is(200)
      ->json_has('/items')
      ->json_has('/items/0/id')
      ->json_has('/items/0/name')
      ->json_has('/items/0/comment')
      ->json_has('/items/0/model')
      ->json_has('/items/0/query_params')
      ->json_has('/items/0/template');

    # export
    unlink("/var/tmp/boxes.json");
    $t->get_ok("/api/v1/admin/repository/export?target=boxes")
      ->status_is(200);
    ok(-e "/var/tmp/boxes.json", "boxes.json generated");

    # import
    $t->get_ok("/api/v1/admin/repository/import?target=boxes")
      ->status_is(200);
    unlink("/var/tmp/boxes.json");

    # delete
    $t->post_ok('/api/v1/admin/sidebar' => {Accept => '*/*'} => json => { action => "delete", id => $id })
      ->status_is(200);

    $t->get_ok("/api/v1/admin/sidebar?id=$id")
      ->status_is(404)
      ->json_hasnt('/item');

    $t->get_ok('/api/v1/admin/sidebar')->status_is(200);
    #$t->get_ok('/admin/sidebar')->status_is(200);

    # finish
    $t->get_ok('/my/logout')->status_is(302);
};

$test_man->cleanup;

done_testing();
