# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::Test qw(create_admin_user create_user delete_user create_story delete_story);

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');

my $story;

if ($t->app->mode eq 'test') {
    my $user = create_user($t->app, "test01", "foobar");
    ok($user, "create test user");
    $story = create_story($t->app, $user, title => 'This is test', body => '<p>This is test!</p>');
    ok($story, "create story for test");
}

SKIP: {
    skip "mode is not 'test'", 4 if ($t->app->mode ne 'test');

    subtest 'comment preview' => sub {
        my $test_data = {
                         title => "テストコメント",
                         comment => "コメントテスト本文\nてすとてすと" . localtime,
                         discussion_id => $story->{discussion_id},
                         stoid => $story->{stoid},
                         action => 'preview',
                        };
        $t->post_ok('/api/v1/comment' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->content_type_like(qr/application\/json/)
          ->json_has('/title')
          ->json_has('/comment');
    };

    subtest 'comment post/retrive' => sub {
        my $test_data = {
                         title => "テストコメント",
                         comment => "コメントテスト本文\nてすとてすと" . localtime,
                         discussion_id => $story->{discussion_id},
                         stoid => $story->{stoid},
                         action => 'post',
                        };
        $t->post_ok('/api/v1/comment' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "comment");

        my $cid = $t->tx->res->json->{cid};
        $t->get_ok("/api/v1/comment?comment_id=$cid")
          ->status_is(200)
          ->json_has('/comments/0/id')
          ->json_hasnt('/comments/0/ipid')
          ->json_is('/comments/0/content_type' => "comment");

    };


}

if ($t->app->mode eq 'test') {
    delete_user($t->app, "test01");
    delete_story($t->app, $story->{stoid});
}


done_testing();
