# -*-Perl-*-
# test for feeds related api
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";

my $feeds = $t->app->model('feeds');
ok($feeds, "get Feeds instance");

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("apipolltestadmin", $test_passwd);
    ok($admin, "create admin user");
}

subtest 'create/select/update/delete/exports feeds and items' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $test_title = "テストのFeed";
    my $test_url = "https://mag.osdn.jp/rss";
    my $test_name = "test_feed";

    # prepare for test
    my $rs = $feeds->select(url => $test_url);
    $feeds->delete(feed_id => $rs->{feed_id}) if $rs;

    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => $admin->{nickname}, passwd => $test_passwd})
      ->status_is(302);

    # fetch_url
    my $params = { action => "fetch",
                   url => $test_url };
    $t->post_ok('/api/v1/admin/feed' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_has('/item/url')
      ->json_has('/item/title')
      ->json_has('/item/description');
    my $new_url = $t->tx->res->json("/item/url");
    my $new_title = $t->tx->res->json("/item/title");
    my $new_description = $t->tx->res->json("/item/description");
    ok($new_url, "has url");
    ok($new_title, "has title");
    ok($new_description, "has description");

    # create
    $params = { action => "create",
                name => $test_name,
                url => $new_url,
                title => $new_title,
                description => $new_description,
              };
    $t->post_ok('/api/v1/admin/feed' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_has('/feed_id');
    my $feed_id = 0;
    if ($t->tx->res->json) {
        if ($t->tx->res->json->{message}) {
            diag $t->tx->res->json->{message};
        }
        $feed_id = $t->tx->res->json->{feed_id};
    }

    # list
    $t->get_ok('/api/v1/admin/feed')
      ->status_is(200)
      ->json_has('/items');

    # get
    $t->get_ok("/api/v1/admin/feed?feed_id=$feed_id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/feed_id' => $feed_id);

    # update
    $params = { action => "update",
                feed_id => $feed_id,
                url => $new_url,
                title => "新しいタイトル",
                description => "新しい説明",
              };
    $t->post_ok('/api/v1/admin/feed' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_is('/feed_id', $feed_id);

    # check updated item
    $t->get_ok("/api/v1/admin/feed?feed_id=$feed_id")
      ->status_is(200)
      ->json_has('/item')
      ->json_is('/item/title' => "新しいタイトル")
      ->json_is('/item/description' => "新しい説明");

    # export
    unlink("/var/tmp/feeds.json");
    $t->get_ok("/api/v1/admin/repository/export?target=feeds")
      ->status_is(200);
    ok(-e "/var/tmp/feeds.json", "feeds.json generated");

    # import
    $t->get_ok("/api/v1/admin/repository/import?target=feeds")
      ->status_is(200);
    unlink("/var/tmp/feeds.json");

    # delete
    $params = { action => "delete",
                feed_id => $feed_id
              };
    $t->post_ok('/api/v1/admin/feed' => {Accept => '*/*'} => json => $params)
      ->status_is(200)
      ->json_is('/error', 0);

    # check deleted item
    $t->get_ok("/api/v1/admin/feed?feed_id=$feed_id")
      ->status_is(404);

    # finish
    $t->get_ok('/my/logout')->status_is(302);
};

$test_man->cleanup;

done_testing();
