# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Mojo::Util qw(dumper);
use Newslash::Util::Test qw(create_user delete_user);

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');

if ($t->app->mode eq 'test') {
    my $user = create_user($t->app, "test01", "foobar");
    ok($user, "create test user");
}

SKIP: {
    skip "mode is not 'test'", 0 if ($t->app->mode ne 'test');

    subtest 'journal post' => sub {
        my $text = <<"EOT";
東芝が2013年7月から2015年8月までに発売されたノートPC45機種で使われているバッテリーの一部について、使い方によっては過熱してショートし発火する恐れがあるとし、新品への無償交換を行うことを発表した（<a href="http://www3.nhk.or.jp/news/html/20161110/k10010763661000.html">NHK</a>、<a href="http://pc.watch.impress.co.jp/docs/news/1029272.html">PC Watch</a>）。

東芝のノートPCについては、<a href="http://hardware.srad.jp/story/14/11/14/2013216/">2014年</a>および<a href="https://mobile.srad.jp/story/16/01/29/2321242/">今年1月</a>にバッテリーの過熱・発煙・発火の恐れがあるとしてバッテリーのリコールが行われているが、発見されたのはこれとは別の不具合とのこと（<a href="https://batterycheck.toshiba.com/BatteryUpdate/InformationJapan?region=TJPN&amp;country=JP&amp;lang=ja">東芝の発表</a>）。そのため、以前問題ないとされたPCについても、再度確認が必要とのこと。

EOT

        my $post_type = 1;
        my $test_data = {
                         item => {
                                  title => "テスト日記",
                                  intro_text => $text,
                                  comment_status => 'enabled',
                                  post_type => $post_type,
                                  formatter => "modern",
                                  #                         tid => '',
                                  # submissioncopy = 0,
                                 },
                         action => 'post',
                        };
        # anonymous user cannot create journals
        $t->post_ok('/api/v1/journal' => {Accept => '*/*'} => json => $test_data)
          ->status_isnt(200);

        # login
        $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => 'test01', passwd => 'foobar'})
          ->status_is(302);

        # login user can create journals
        $t->post_ok('/api/v1/journal' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "journal")
          ->json_hasnt('/message')
          ->or(sub {diag "message: " . $t->tx->res->json->{message}});


        # update journal
        my $id = $t->tx->res->json->{id};
        $test_data = {
                      item => {
                               id => $id,
                               title => "テスト日記（updated）",
                               intro_text => $text . " - updated",
                               comment_status => 'enabled',
                               post_type => $post_type,
                               formatter => "modern",
                               #                         tid => '',
                               # submissioncopy = 0,
                              },
                      action => 'post',
                     };
        $t->post_ok('/api/v1/journal' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "journal")
          ->json_hasnt('/message')
          ->or(sub {diag "message: " . $t->tx->res->json->{message}});

        # logout
        $t->get_ok('/my/logout')->status_is(302);

    };

    subtest 'journal preview' => sub {
        my $test_data = {
                         item => {
                                  title => "テスト日記",
                                  introtext => "日記テスト本文\nてすとてすと" . localtime,
                                  commentstatus => 'enabled',
                                 },
                         action => 'preview',
                        };

        # anonymous user cannot create journals
        $t->post_ok('/api/v1/journal' => {Accept => '*/*'} => json => $test_data)
          ->status_isnt(200);

        # login
        $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => 'test01', passwd => 'foobar'})
          ->status_is(302);

        # login user can create journals
        $t->post_ok('/api/v1/journal' => {Accept => '*/*'} => json => $test_data)
          ->status_is(200)
          ->content_type_like(qr/application\/json/)
          ->json_hasnt('/message')
          ->or(sub {diag "message: " . $t->tx->res->json->{message}});

        # logout
        $t->get_ok('/my/logout')->status_is(302);
    };

};

subtest 'get journal' => sub {
    # login
    #$t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => 'test01', passwd => 'foobar'})
    #  ->status_is(302);

    my $url;
    if ($t->app->mode eq 'test') {
        $url = '/api/v1/journal?nickname=test01';
    }
    else {
        $url = '/api/v1/journal?nickname=hylom';
    }

    $t->get_ok($url)
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/journals')
      ->or(sub {diag "message: " . dumper($t->tx->res->json);});

    # logout
    #$t->get_ok('/my/logout')->status_is(302);
};

if ($t->app->mode eq 'test') {
    delete_user($t->app, "test01");
}


done_testing();
