# -*-Perl-*-
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');
my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("apipolltestadmin", $test_passwd);
    ok($admin, "create admin user");
    $user = $test_man->create_user("polltest", $test_passwd);
    ok($user, "create test user");
}

SKIP: {
    skip "mode is not 'test'", 4 if ($t->app->mode ne 'test');

    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => $admin->{nickname}, passwd => $test_passwd})
      ->status_is(302);

    subtest 'create / post poll' => sub {
        # create
        my $poll_data = {
                         poll => {
                                  question => "投票テスト用質問",
                                  answers => [ "質問1", "質問2", "質問3", "質問4", ],
                                 }
                        };

        $t->post_ok('/api/v1/poll' => {Accept => '*/*'} => json => $poll_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "poll");
        #diag $t->tx->res->json->{message} if $t->tx->res->code != 200;
        my $qid = $t->tx->res->json->{id};

        # vote
        my $vote_data = { vote => {qid => $qid, aid => 1} };
        $t->post_ok('/api/v1/vote' => {Accept => '*/*'} => json => $vote_data)
          ->status_is(200)
          ->json_has('/id')
          ->json_is('/type' => "answer");
        #diag $t->tx->res->json->{message} if $t->tx->res->code != 200;

        # duplicate vote
        $vote_data = { qid => $qid, aid => 2 };
        $t->post_ok('/api/v1/vote' => {Accept => '*/*'} => json => $vote_data)
          ->status_is(400);

        # delete
        my $polls = $t->app->model('polls');
        ok($polls->hard_delete($qid), 'delete polls');

    };

    $t->get_ok('/my/logout')->status_is(302);
}

$test_man->cleanup;

done_testing();
