# -*-Perl-*-
# destructive database-related  tests
use Mojo::Base -strict;
use Mojo::Date;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::TestMan;

use POSIX qw(strftime);
my $t = Test::Mojo->new('Newslash::Web');

my $test_man = Newslash::Util::TestMan->new($t);
my ($admin, $user);
my $test_passwd = "foobar";

if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin("apisubtestadmin", $test_passwd);
    ok($admin, "create admin user");
}

subtest 'create/preview/get submission' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $test_data = {
                     action => 'preview',
                     item => {
                              title => "テストタレコミ @ " . localtime,
                              introtext => "テスト本文\nてすとてすと" . localtime,
                             },
                    };

    # preview
    $t->post_ok('/api/v1/submission' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/item/title')
      ->json_has('/item/intro_text')
      ->json_has('/item/email')
      ->json_has('/item/tid');
    diag $t->tx->res->json->{message} if $t->tx->res->code != 200;
      #print STDERR Dumper

    # post
    $test_data->{action} = 'post';
    $t->post_ok('/api/v1/submission' => {Accept => '*/*'} => json => $test_data)
      ->status_is(200)
      ->json_has('/id')
      ->json_is('/type' => "submission");
    my $sub_id = $t->tx->res->json->{id};
    ok($sub_id, "valid submission id");

    # duplicated submission
    $t->post_ok('/api/v1/submission' => {Accept => '*/*'} => json => $test_data)
      ->status_is(409);

    # get
    $t->get_ok("/api/v1/submission?subid=$sub_id")
      ->status_is(200)
      ->content_type_like(qr/application\/json/)
      ->json_has('/item/title')
      ->json_has('/item/introtext')
      ->json_has('/item/email')
      ->json_has('/item/tid')
      ->json_has('/item/submission_id');
    diag $t->tx->res->json->{message} if $t->tx->res->code != 200;
    is($t->tx->res->json->{item}->{submission_id}, $sub_id, "valid submission id");

    # login as admin
    $t->post_ok('/my/login' => {Accept => '*/*'} => form => {nickname => $admin->{nickname}, passwd => $test_passwd})
      ->status_is(302);

    # get blocked items
    $t->get_ok('/api/v1/admin/blocking?type=submission')
      ->status_is(200)
      ->json_has('/items');
    my $items = $t->tx->res->json->{items};
    ok($items, "get blocked items");
    my $blocks = @$items;

    # block submission
    my $data = { action => "block", type => "submission", id => $sub_id };
    $t->post_ok('/api/v1/admin/blocking' => {Accept => '*/*'} => json => $data)
      ->status_is(200)
      ->json_has('/id')
      ->json_is('/type' => "submission");

    # get blocked items (2)
    $t->get_ok('/api/v1/admin/blocking?type=submission')
      ->status_is(200)
      ->json_has('/items');
    $items = $t->tx->res->json->{items};
    ok($items, "get blocked items (2)");
    is(@$items, $blocks + 1, "number of blocked items is incremented");

    # block submission
    $data->{action} = "unblock";
    $t->post_ok('/api/v1/admin/blocking' => {Accept => '*/*'} => json => $data)
      ->status_is(200)
      ->json_has('/id')
      ->json_is('/type' => "submission");

    # get blocked items (3)
    $t->get_ok('/api/v1/admin/blocking?type=submission')
      ->status_is(200)
      ->json_has('/items');
    $items = $t->tx->res->json->{items};
    ok($items, "get blocked items (3)");
    is(@$items, $blocks, "number of blocked items is decremented");

    # delete item
    my $submissions = $t->app->model('submissions');
    ok($submissions->hard_delete(submission_id => $sub_id), "delete item");
};

$test_man->cleanup;

done_testing();
