# -*-Perl-*-
# Newslash::Model::Comments tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::Test qw(create_user delete_user create_story delete_story);

my $t = Test::Mojo->new('Newslash::Web');

use constant TEST_USER_NAME => 'comment_tester';
my ($user, $story);

# prepare for test
if ($t->app->mode eq 'test') {
    $user = create_user($t->app, TEST_USER_NAME);
    ok($user, "create user for test");

    $story = create_story($t->app, $user, title => 'This is test', body => '<p>This is test!</p>');
    ok($story, "create story for test");
}

subtest 'get comments object' => sub {
    my $comments = $t->app->model('comments');
    ok($comments, 'get comments object');
};

subtest 'create/select/update/delete comments' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');
    my $comments = $t->app->model('comments');
    my $params = {
                  discussion_id => $story->{discussion_id},
                  title => 'comment test',
                  comment => 'comment test body. this is comment test body.',
                 };
    my $cid = $comments->create($params, $user);
    ok($cid, "comment create");
    diag($comments->last_error) if !$cid;

    my $comment = $comments->select(cid => $cid,
                                    security_level => $user->{seclev});
    ok($comment, "select comment");
    is($comment->{cid}, $cid, "select valid comment");

    $params->{pid} = $cid;
    $params->{title} = 'Re: comment test';
    my $cid2 = $comments->create($params, $user);
    ok($cid2, "reply comment create");

    ok($comments->update(cid => $cid, points => 2), "comment points update");

    $comment = $comments->select(order_by => { points => "DESC" },
                                 months => 12,
                                 security_level => $user->{seclev});
    ok($comments, "select multiple items");

    my $dt = DateTime->now();
    my $c;
    my $year = $dt->year;
    my $month = $dt->month;
    my $day = $dt->day;
    my $offset = $user->{config}->{offset_sec} || 0;

    $c = $comments->count(year => $year,
                             month => $month,
                             day => $day,
                             offset_sec => $offset);
    ok($c, "get count of the day");
    ok($c > 0, "valid count of the day");

    $c = $comments->count(year => $year,
                             month => $month,
                             offset_sec => $offset);
    ok($c, "get count of the month");
    ok($c->{$day}, "valid count of the month");

    $c = $comments->count(year => $year,
                             offset_sec => $offset);
    ok($c, "get count of the year");
    ok($c->{$month}, "valid count of the year");

    ok($comments->hard_delete(cid => $cid2), "delete comment $cid2");
    ok($comments->hard_delete(cid => $cid), "delete comment $cid");

};

# finish
if ($t->app->mode eq 'test') {
    delete_story($t->app, $story->{stoid});
    delete_user($t->app, TEST_USER_NAME);
}

done_testing();
