# -*-Perl-*-
# Newslash::Model::Filters tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;

my $t = Test::Mojo->new('Newslash::Web');

use constant TEST_FILTER_NAME => "testfilter";

# prepare for test
if ($t->app->mode eq 'test') {
    my $filters = $t->app->model('filters');
    my $filter = $filters->select(name => TEST_FILTER_NAME);
    if ($filter) {
        my $rs = $filters->delete($filter->{filter_id});
    }
}

subtest 'get filters object' => sub {
    my $filters = $t->app->model('filters');
    ok($filters, 'get filtersboxes object');
};

subtest 'create/select/update/delete filters' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $filters = $t->app->model('filters');
    my $params = {
                  name => TEST_FILTER_NAME,
                  description => "Test Filter",
                  message => "filter test",
                  regexp => '1\s*[Gg]et',
                  ratio => 1.0,
                  threshold => 0,
                  limit => 0,
                  pathes => ['^\/api\/v1\/comment$', ],
                  fields => ["/title", "/comment"],
                 };
    my $filter_id = $filters->create(%$params);
    ok($filter_id, 'create filter');

    my $filter1 = $filters->select(filter_id => $filter_id);
    ok($filter1, 'select filter by id');
    is($filter1->{filter_id}, $filter_id, 'select valid filter');

    my $filter2 = $filters->select(name => $params->{name});
    ok($filter2, 'select filter by name');
    is($filter2->{filter_id}, $filter_id, 'select valid filter');

    my $filter3 = $filters->select;
    ok($filter3, 'select all filters');
    ok(@$filter3 > 0, 'select one or more filtere(s)');

    my $new_comment = 'Updated filter comment';
    $params->{description} = $new_comment;
    my $rs = $filters->update(filter_id => $filter_id, %$params);
    ok($rs, 'update filter');
    my $new_filter = $filters->select(filter_id => $filter_id);
    is($new_filter->{description}, $new_comment, 'update succeed');

    $rs = $filters->delete(filter_id => $filter_id);
    ok($rs, 'delete filter');
    $new_filter = $filters->select(filter_id => $filter_id);
    ok(!defined $new_filter, 'delete succeed');
};

done_testing();
