# -*-Perl-*-
# Newslash::Model::Firehose related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use DateTime;

use Newslash::Util::TestMan;

my $t = Test::Mojo->new('Newslash::Web');
my $stories = $t->app->model('stories');
ok($stories, "get Stories instance");
my $firehose = $t->app->model('firehose');
ok($firehose, "get Firehose instance");
my $tags = $t->app->model('tags');
ok($tags, "get Tags instance");

use constant TEST_USER_NAME => 'firehose_tester';
my ($admin, $story);
my $test_man = Newslash::Util::TestMan->new($t);

# prepare for test
if ($t->app->mode eq 'test') {
    $admin = $test_man->create_admin(TEST_USER_NAME);
    ok($admin, "create admin user");

    $story = $test_man->create_story( user => $admin,
                                      uid => $admin->{uid},
                                      title => "test story for firehose",
                                      dept => "test for firehose",
                                      introtext => '<p>foo bar hoge</p>',
                                      bodytext => '',
                                      topics_chosen => {49 => 10, 2271 => 20},
                                      commentstatus => '',
                                      tid => 49,
                                      time => "2020-11-11 10:00:00");
    ok($story, "create story for test");
}

subtest 'then select from firehose' => sub {
    plan skip_all => "mode is not 'test'" if ($t->app->mode ne 'test');

    my $item = $firehose->select(type => "story",
                                 order_by => {id => "DESC"},
                                );
    ok($item, "select firehose item");
    ok(@$item, "select valid number of firehose item");
    is($item->[0]->{srcid}, $story->{stoid}, "get valid item");

    # set tag for item
    my $rs =$tags->set(globj_id => $item->[0]->{globjid},
                       tag_names => ["hoge", "moge", "abcdefg"]);
    ok($rs, "set tag");
    diag $tags->last_error if !$rs;

    # select by tag
    $item = $firehose->select(type => "story",
                              tag_name => "abcdefg",
                              order_by => {id => "DESC"},
                             );
    ok($item, "select firehose item by tag");
    ok(@$item, "select valid number of firehose item by tag");
    is($item->[0]->{srcid}, $story->{stoid}, "get valid item");
    ok(@{$item->[0]->{tags}} > 2, "get valid number of tags");

    # update item
    my $new_title = "test story new item";
    my $new_dept = "new dept";
    $rs = $firehose->update(firehose_id => $item->[0]->{firehose_id},
                            title => $new_title,
                            dept => $new_dept);
    ok($rs, "update firehose item");
    diag($firehose->last_error) if !$rs;

    $item = $firehose->select(firehose_id => $item->[0]->{firehose_id});
    is($item->{title}, $new_title, "title updated");
    is($item->{dept}, $new_dept, "dept updated");

    #diag(Dumper($item->[0]))
};

$test_man->cleanup;

done_testing();
