# -*-Perl-*-
# Newslash::Model::Metamoderations related  tests
use Mojo::Base -strict;

use Test::More;
use Test::Mojo;
use Data::Dumper;
use Newslash::Util::Test qw(create_user delete_user
                            create_story delete_story
                            create_comments
                            create_users create_admin_user
                            get_anonymous_user
                          );


use POSIX qw(strftime);

my $t = Test::Mojo->new('Newslash::Web');

my $moderations = $t->app->model('moderations');
ok($moderations, "get Moderations instance");
my $metamoderations = $t->app->model('metamoderations');
ok($metamoderations, "get Metamoderations instance");
my $users = $t->app->model('users');
ok($users, "get Users instance");
my $comments = $t->app->model('comments');
ok($comments, "get Comments instance");
my $stories = $t->app->model('stories');
ok($stories, "get Stories instance");

my $test_users;
my $story;
my $admin;
my $anonymous;

# prepare for test
if ($t->app->mode eq 'test') {
    # at first, create test users.
    $test_users = create_users($t->app, "mmtest", 10);
    $admin = create_admin_user($t->app, "mmtest_admin");

    # second, create story.
    $story = create_story($t->app, $admin, "メタモデテスト", "<p>メタモデテスト本文</p>");
    ok($story, "create story for test");

    $anonymous = get_anonymous_user($t->app);
    ok($anonymous, "get anonymous user");
}

SKIP: {
    skip "mode is not 'test'", 1 if ($t->app->mode ne 'test');
    # add some parameter to emulate web access
    my $discussion_id = $story->{discussion};

    # create comments
    my $comments = [
                    {
                     key => "01",
                     title => "コメントテスト01",
                     comment => "コメントテスト01の本文です。",
                     author => $anonymous,
                     children => [
                                  {
                                   key => "02",
                                   title => "Re: コメントテスト01",
                                   comment => "コメントテスト02の本文です。コメントテスト01の子コメントです。",
                                   author => $test_users->{mmtest1},
                                  },
                                 ],
                    },
                    {
                     key => "03",
                     title => "コメントテスト03",
                     comment => "コメントテスト03の本文です。",
                     author => $test_users->{mmtest2},
                     children => [
                                  {
                                   key => "04",
                                   title => "コメントテスト04",
                                   comment => "コメントテスト04の本文です。コメントテスト03の子コメントです。",
                                   author => $test_users->{mmtest3},
                                  },
                                 ],
                    },
                    {
                     key => "05",
                     title => "コメントテスト05",
                     comment => "コメントテスト05の本文です。",
                     author => $test_users->{mmtest4},
                    },
                   ];
    my $test_comments = create_comments($t->app, $discussion_id, $comments);
    ok($test_comments, "create comments");
    #diag(Dumper($test_comments));
    # get moderation reasons
    #my $reasons = $moderations->reasons;
    #ok($reasons, "get reasons");

    # moderation comment02
    my $rs = $moderations->create($admin, $test_comments->{"02"}->{cid}, 8); # id 8 is おもしろおかしい
    if (!$rs) {
        diag($moderations->last_error);
    }
    ok($rs, "plus moderation");

    # moderation comment03
    if (!$rs) {
        diag($moderations->last_error);
    }
    $rs = $moderations->create($admin, $test_comments->{"05"}->{cid}, 3); # id 3 is 荒し
    ok($rs, "minus moderation");

    # finished. do cleanup...
    # delete test users
    #for my $n (0..9) {
    #    my $name = "mmtest" . $n;
    #    my $rs = $users->hard_delete($test_users->{$name}->{uid});
    #    ok($rs, "delete user $name");
    #}

}

# cleanup
if ($t->app->mode eq 'test') {
    for my $u (keys %$test_users) {
        delete_user($t->app, $u);
    }
    delete_user($t->app, $admin->{nickname});
    delete_story($t->app, $story->{stoid});
}

done_testing();
