#!/usr/bin/python

import newslash_db
import lucene_wrapper

from yaml import load
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

CONFIG_FILE="./.config.yml"

def main():
    config = load_config(CONFIG_FILE)
    db_cfg = config["Database"]

    stories = newslash_db.Stories(db_cfg)
    items = stories.select(limit=10)

    lucene_wrapper.init_vm()
    with lucene_wrapper.Indexer(index_directory="./index") as indexer:
        for item in items:
            doc = lucene_wrapper.Document()
            doc.add_string_field("type", "story")
            doc.add_string_field("id", item["sid"])
            doc.add_text_field("content_text", item["introtext"] + item["bodytext"])
            print("index {}...".format(item["sid"]))
            indexer.add(doc)


def load_config(pathname):
    fh = open(pathname)
    config = load(fh, Loader=Loader)
    fh.close()
    return config


if __name__ == "__main__":
    main()
