# coding: utf-8
'''stories.py: newslash firehose related database module'''

from base import NewslashDB, NewslashDBError

CREATE_FIREHOSE_TABLE_SQL = '''
CREATE TABLE IF NOT EXISTS `firehose` (
  `id`               int(8) unsigned                                                                                                   NOT NULL AUTO_INCREMENT,
  `uid`              mediumint(8) unsigned                                                                                             NOT NULL DEFAULT '0',
  `globjid`          int(10) unsigned                                                                                                  NOT NULL DEFAULT '0',
  `discussion`       mediumint(8) unsigned                                                                                             NOT NULL DEFAULT '0',
  `type`             enum('submission','journal','bookmark','feed','story','vendor','misc','comment','discussion','project','tagname') DEFAULT 'submission',
  `createtime`       datetime                                                                                                          NOT NULL DEFAULT '1970-01-01 00:00:00',
  `popularity`       float                                                                                                             NOT NULL DEFAULT '0',
  `editorpop`        float                                                                                                             NOT NULL DEFAULT '0',
  `neediness`        float                                                                                                             NOT NULL DEFAULT '0',
  `activity`         float                                                                                                             NOT NULL DEFAULT '0',
  `accepted`         enum('no','yes')                                                                                                  DEFAULT 'no',
  `rejected`         enum('no','yes')                                                                                                  DEFAULT 'no',
  `public`           enum('no','yes')                                                                                                  DEFAULT 'no',
  `attention_needed` enum('no','yes')                                                                                                  DEFAULT 'no',
  `is_spam`          enum('no','yes')                                                                                                  DEFAULT 'no',
  `bayes_spam`       enum('no','yes')                                                                                                  DEFAULT 'no',
  `collateral_spam`  enum('no','yes')                                                                                                  DEFAULT 'no',
  `primaryskid`      smallint(5)                                                                                                       DEFAULT '1',
  `tid`              int(10) unsigned                                                                                                  DEFAULT NULL,
  `srcid`            int(10) unsigned                                                                                                  NOT NULL DEFAULT '0',
  `url_id`           int(8) unsigned                                                                                                   NOT NULL DEFAULT '0',
  `toptags`          varchar(255)                                                                                                      DEFAULT '',
  `email`            varchar(255)                                                                                                      NOT NULL DEFAULT '',
  `emaildomain`      varchar(255)                                                                                                      NOT NULL DEFAULT '',
  `name`             varchar(50)                                                                                                       NOT NULL,
  `dept`             varchar(100)                                                                                                      NOT NULL DEFAULT '',
  `ipid`             varchar(32)                                                                                                       NOT NULL DEFAULT '',
  `subnetid`         varchar(32)                                                                                                       NOT NULL DEFAULT '',
  `category`         varchar(30)                                                                                                       NOT NULL DEFAULT '',
  `nexuslist`        varchar(32)                                                                                                       NOT NULL DEFAULT '',
  `last_update`      timestamp                                                                                                         NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  `signoffs`         varchar(255)                                                                                                      NOT NULL DEFAULT '',
  `stoid`            mediumint(8) unsigned                                                                                             DEFAULT '0',
  `body_length`      mediumint(8) unsigned                                                                                             NOT NULL DEFAULT '0',
  `word_count`       mediumint(8) unsigned                                                                                             NOT NULL DEFAULT '0',
  `srcname`          varchar(32)                                                                                                       NOT NULL DEFAULT '',
  `thumb`            mediumint(8) unsigned                                                                                             DEFAULT NULL,
  `mediatype`        enum('text','none','video','image','audio')                                                                       NOT NULL DEFAULT 'none',
  `offmainpage`      enum('no','yes')                                                                                                  NOT NULL DEFAULT 'no',
  `sprite`           varchar(128)                                                                                                      NOT NULL DEFAULT '',
  `sprite_info`      mediumtext                                                                                                        NOT NULL,
  `preview`          enum('no','yes')                                                                                                  DEFAULT 'no',
  PRIMARY KEY               (`id`),
  UNIQUE KEY                `globjid` (`globjid`),
  KEY `createtime`          (`createtime`),
  KEY `popularity`          (`popularity`),
  KEY `url_id`              (`url_id`),
  KEY `neediness`           (`neediness`),
  KEY `uid`                 (`uid`),
  KEY `last_update`         (`last_update`),
  KEY `type_srcid`          (`type`,`srcid`),
  KEY `ipid`                (`ipid`),
  KEY `ibfk_converttid_15`  (`tid`),
  KEY `uid_createtime_type` (`uid`,`createtime`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 ROW_FORMAT=DYNAMIC
'''

class Firehose(NewslashDB):
    def __init__(self, config):
        self.db = NewslashDB(config)
        self._config = config

    def create_tables(self):
        '''this function implemented for test purpose...'''
        self.db.execute_multi((CREATE_FIREHOSE_TABLE_SQL,))

    def drop_tables(self):
        '''this function implemented for test purpose...'''
        self.db.execute_multi(('DROP TABLE firehose',))

