#!/usr/bin/python
# -*- coding: utf-8
from __future__ import print_function

import sys
import os.path

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from newslash_index import Index, DatabaseError, Metadata, MySQLMetadata

from yaml import load
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

CONFIG_FILE = '../searchd.conf'
OUTPUT_DIR = '.'

def _load_config(pathname):
    fh = open(pathname)
    config = load(fh, Loader=Loader)
    fh.close()
    return config

    
def main():
    config = _load_config(CONFIG_FILE)

    # get old metadata from MySQL DB
    metadata_old = MySQLMetadata(dict(database=config["Database"]))
    data = metadata_old.get_all()

    print("==== metadata: ====")
    for item in data:
        print(item)
    
    # put metadata to file
    metadata = Metadata(dict(index_path=OUTPUT_DIR))

    print("create table...")
    metadata.create_table()
    print("ok")
    
    for item in data:
        sys.stdout.write(".")
        sys.stdout.flush()
        metadata.update(item["target_name"], item["last_update"], item["latest_id"])

    print("convert done.")
    new_data = metadata.get_all()
    print("==== new metadata: ====")
    for item in new_data:
        print(item)
    

if __name__ == '__main__':
    main()

