#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use feature ':5.10';

use Getopt::Std;
use DBI;
use File::Basename;
use File::Spec;

my $PROGNAME = basename($0);
my $VERSION = '0.1.0';
my $SQL_DIR = './sqls';
my $DATA_DIR = './data';

sub recreate_database {
    my $opts = shift;
    my ($sql, $sth, $result);
    my $host = $opts->{'H'} || "";
    my $user = $opts->{'u'} || "";
    my $password = $opts->{'p'} || "";
    my $name = $opts->{'d'} || "";

    my $k = "";
    print "DROP and recreate database '$name'. ok? (y/N) ";
    $k = <STDIN>;
    chomp $k;
    if ($k ne "y" && $k ne "Y") {
        die "abort.";
    }

    my $dbh = DBI->connect("DBI:mysql::$host", $user, $password);

    print "drop database '$name' ...\n";
    $sql = "DROP DATABASE $name;";
    $sth = $dbh->prepare($sql);
    $result = $sth->execute;
    if (!$result) {
	print "sql execute error: $!\n";
    }
    $sth->finish;

    $sql = "CREATE DATABASE $name DEFAULT CHARACTER SET utf8mb4";
    $sth = $dbh->prepare($sql);
    $result = $sth->execute;
    if (!$result) {
	print "sql execute error: $!\n";
    }
    $sth->finish;
}

sub main {
  my $opts = {};

  if (!getopts( "hvru:H:p:d:", $opts)) {
    usage('Options used incorrectly.');
  }

  if ($opts->{'h'}) {
    usage();
  }

  if ($opts->{'r'}) {
      recreate_database($opts);
  }

  my $dbh = connect_db($opts);
  if (!$dbh) {
    usage('cannot connect to database.');
  }
  print "create databases...\n";
  exec_sql($dbh, $SQL_DIR);

  #print "insert datas...\n";
  #exec_sql($dbh, $DATA_DIR);

  $dbh->disconnect();
  print "done.\n";
}

sub exec_sql {
    my $dbh = shift;
    my $target = shift;

    opendir(my $dh, $target) || die "cannot opendir $target: $!\n";
    my @files;
    while (my $fname = readdir $dh) {
        if ($fname =~ m/^\./) {
            next;
        }
        if ($fname =~ m/^.*\.sql$/) {
            push @files, $fname;
        }
    }
    @files = sort @files;

    while (my $fname = shift @files) {
        print "execute $fname...\n";
        my $sql = get_sql_contents($target, $fname);
        # my $sth = $dbh->prepare($sql);
        # my $result = $sth->execute;
        my $result = $dbh->do($sql);
        if (!$result) {
            print "sql execute error: $!\n";
        }
    }
}


sub get_sql_contents {
    my $dirname = shift @_;
    my $fname = shift @_;

    my $pathname = File::Spec->catfile($dirname, $fname);
    my $fh;

    if (!open($fh, '<', $pathname)) {
        print "cannot open $fh: $!\n";
        return;
    }

    my $sql = do { local $/; <$fh> };
    return $sql;
}


sub connect_db {
  my $opts = shift @_;

  my $host = $opts->{'H'} || "";
  my $user = $opts->{'u'} || "";
  my $password = $opts->{'p'} || "";
  my $name = $opts->{'d'} || "";

  my $dbh = DBI->connect("DBI:mysql:$name:$host", $user, $password);
  return $dbh;
}

sub usage {
  print "*** $_[0] \n" if $_[0];
  print <<EOT;

Usage: $PROGNAME [OPTIONS] <database name>

This will create table in MySQL database.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
      -r    Recreate database
EOT
  exit;
}

sub version {
  print <<EOT;


$PROGNAME $VERSION
EOT
  exit;
}

main();

__END__
