#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use feature ':5.10';

use Getopt::Std;
use DBI;
use File::Basename;
use File::Spec;
use Data::Dumper;

my $PROGNAME = basename($0);
my $VERSION = '0.1.0';
my $SQL_DIR = './sqls';

sub main {
  my $opts = {};

  if (!getopts( "hvu:H:p:d:t:", $opts)) {
    usage('Options used incorrectly.');
  }

  if ($opts->{'h'}) {
    usage();
  }

  my $table = $opts->{t};
  if (!$table) {
      usage('no table given.');
  }

  my $dbh = connect_db($opts);
  if (!$dbh) {
    usage('cannot connect to database.');
  }

  my $sql = "SHOW CREATE TABLE $table";
  my $sth = $dbh->prepare($sql);
  my $result = $sth->execute;
  if (!$result) {
      print "sql execute error: $!\n";
  }
  my $rs = $sth->fetchrow_hashref();
  $sth->finish;
  $dbh->disconnect();
  print "$rs->{'Create Table'}\n";
}

sub connect_db {
  my $opts = shift @_;

  my $host = $opts->{'H'} || "";
  my $user = $opts->{'u'} || "";
  my $password = $opts->{'p'} || "";
  my $name = $opts->{'d'} || "";

  my $dbh = DBI->connect("DBI:mysql:$name:$host", $user, $password);
  return $dbh;
}

sub usage {
  print "*** $_[0] \n" if $_[0];
  print <<EOT;

Usage: $PROGNAME [OPTIONS]

This will create table SQL file to create table.

Main options:
      -h    Help (this message)
      -v    Version
      -u    Username to use when connecting to server
      -H    Connect to host
      -p    Password to use when connecting to server
      -d    The database to use
      -t    Table name to get SQL
EOT
  exit;
}

sub version {
  print <<EOT;


$PROGNAME $VERSION
EOT
  exit;
}

main();

__END__
